#!/bin/bash

#-------------------------------------------------------------
# SCRIPT PARA A PRIMEIRA CONFIGURAÇÃO FTP
# 
# - Adiciona um novo shell
# - Adiciona um novo usuário FTP wegftp
# - Cria os diretórios necessários
# - Modifica as permissões necessárias para a escrita no diretório
# - Monta a pasta compartilhada entre o usuário e a pasta ftp em codesys 
# - Reinicia o FTP
# - Cria a chave utilizada para o FTPS
#
#
# script localizado em: /opt/weg/scripts/ftp/ 
#
# Version: 1.2
#
#------------------------------------------------------------- 

#-------------------------------------------------------------
# Cria o arquivo Json
#-------------------------------------------------------------
plctools --ftp init;

#-------------------------------------------------------------
# Remove a configuração padrão do FTP
#-------------------------------------------------------------
systemctl stop vsftpd;
rm /etc/vsftpd.conf;

#-------------------------------------------------------------
# Pega o login do FTP do arquivo Json
#-------------------------------------------------------------
#ftp_login=$(grep -oP '(?<="FTP_LOGIN": ")[^ ]*(?=")' /etc/weg/ftp/ftpcfg.json);                                        
ftp_login="wegftp"
if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi

#-------------------------------------------------------------
# Restringe a permisão do usuário ftp
#-------------------------------------------------------------
sed -i '/false/d' /etc/shells                                 
if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi  

echo "/bin/false" >> /etc/shells
if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi
        
#-------------------------------------------------------------
# Pega senha do usuário FTP do arquivo Json
#-------------------------------------------------------------
ftp_passwd=$(grep -oP '(?<="FTP_PASS": ")[^ ]*(?=")' /etc/weg/ftp/ftpcfg.json);                                        
if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi

#-------------------------------------------------------------
# Testa se o login e a senha não são nulos
#-------------------------------------------------------------
if [[ -z "$ftp_login" || -z "$ftp_passwd" ]];
then
    echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat;
else 
    #-------------------------------------------------------------
    # Cria o usuário FTP no linux
    #-------------------------------------------------------------
    echo -e "$ftp_passwd\n$ftp_passwd" | adduser $ftp_login --gecos "" --shell /bin/false --force-badname;                                                                      
    if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi  

    #-------------------------------------------------------------
    # Cria o diretório do usuário se não existir
    #-------------------------------------------------------------
    if [[ ! -d "/home/$ftp_login/ftp" ]];
    then
        mkdir -p /home/$ftp_login/ftp;                                              
        if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi  
    fi

    #-------------------------------------------------------------
    # Altera a permissão da pasta
    #-------------------------------------------------------------
    chown nobody:nogroup /home/$ftp_login/ftp;                         
    if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi  

    chmod a-w /home/$ftp_login/ftp;                                       
    if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi  

    #-------------------------------------------------------------
    # Cria o diretório ftp dentro da pasta codesys se não existir
    #-------------------------------------------------------------
    if [[ ! -d "/var/lib/weg/codesys/ftp" ]];
    then
        mkdir -p /var/lib/weg/codesys/ftp;                                         
        if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi  
    fi

    #-------------------------------------------------------------
    # Cria o diretório que será compartilhado com o usuário ftp
    #-------------------------------------------------------------
    if [[ ! -d "/home/$ftp_login/ftp/files" ]];
    then
        mkdir -p /home/$ftp_login/ftp/files;
        if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi  
    fi

    #-------------------------------------------------------------
    # Cria um link entre as duas pastas
    #-------------------------------------------------------------
    mount --bind /var/lib/weg/codesys/ftp /home/$ftp_login/ftp/files;
    if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi      

    #-------------------------------------------------------------
    # Altera a permissão da pasta
    #-------------------------------------------------------------
    chown $ftp_login:$ftp_login /home/$ftp_login/ftp/files;                  
    if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi  

    chmod -R 755 /home/$ftp_login/ftp/files;
    if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi                                   

    #-------------------------------------------------------------
    # Adiciona o login na lista FTP
    #-------------------------------------------------------------
    echo "$ftp_login" | tee -a /etc/vsftpd.userlist;
    if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi  

    #-------------------------------------------------------------
    # Cria uma chave para o usuário FTPs
    #-------------------------------------------------------------
    openssl req -x509 -nodes -days 18250 -newkey rsa:2048 -keyout /etc/ssl/private/vsftpd.pem -out /etc/ssl/private/vsftpd.pem -subj "/C=BR/ST=Santa Catarina/L=Jaragua do Sul/O=WEG/CN=PLC"
    if [ $? -ne 0 ]; then echo "plclogFTP: $0 FAULT $LINENO" | systemd-cat; fi
fi  
                                        

