#!/bin/bash

#-------------------------------------------------------------
# SCRIPT DEFINIÇÃO DE VARIÁVEIS E FUNÇÕES
# 
# Julho/2023
# 
# - Cores para prints de logs
# - Função logtime
# - Função baseName
# - Função plcLog
#
#-------------------------------------------------------------


#-------------------------------------------------------
# Directories
#-------------------------------------------------------
logdir='/var/log/weg/plcLogs.log'
logdirOthers='/var/log/weg/plcLogsOthers.log'


#-------------------------------------------------------
# Colors
#-------------------------------------------------------
RED='\e[1;41m'
YELLOW='\e[1;43m'
GREEN='\e[1;42m'
BLUE='\e[1;44m'
MAG='\e[1;45m'
CYAN='\e[1;46m'
GRAY='\e[1;100m'
BOLD='\033[1m'
END='\e[0m'


#-------------------------------------------------------
# logtime Function
#-------------------------------------------------------
function logtime {
   date +'%Y-%m-%d %T'
}

#-------------------------------------------------------
# baseName Function
#-------------------------------------------------------
function baseName {
    baseTeste=$(basename "$0" 2> /dev/null)

    # Verifica se a variável baseTeste não está vazia
    if [ -n "$baseTeste" ]; 
    then
        basename "$0"
    else
        echo "terminal"
    fi
}

#-------------------------------------------------------
# plcLog Function
#-------------------------------------------------------
function plcLog {

    case "$1" in
            "INFO")
                if [ $2 == '--newbase' ]; 
                then newbase=$3
                     printf "$(logtime) $GREEN$BOLD I $END $HOSTNAME %-15s %-4s ${*:4}\n" $newbase $BASH_LINENO >> "$logdir"
                else printf "$(logtime) $GREEN$BOLD I $END $HOSTNAME %-15s %-4s ${*:2}\n" $(baseName) $BASH_LINENO >> "$logdir"
                fi
                ;;

            "ERROR")
                if [ $2 == '--newbase' ]; 
                then newbase=$3
                     printf "$(logtime) $RED$BOLD E $END $HOSTNAME %-15s %-4s ${*:4}\n" $newbase $BASH_LINENO >> "$logdir"
                else printf "$(logtime) $RED$BOLD E $END $HOSTNAME %-15s %-4s ${*:2}\n" $(baseName) $BASH_LINENO >> "$logdir"
                fi
                ;;

            "WARN")
                printf "$(logtime) $YELLOW$BOLD W $END $HOSTNAME %-15s %-4s ${*:2}\n" $(baseName) $BASH_LINENO >> "$logdir"
                ;;

            "SSH")
                if [ "$USER" == "root" ] 
                then printf "$(logtime) $YELLOW$BOLD W $END $HOSTNAME ssh root             ssh $SSH_CONNECTION \n" >> "$logdir"
                else printf "$(logtime) $YELLOW$BOLD W $END $HOSTNAME ssh $USER          ssh $SSH_CONNECTION \n" >> "$logdirOthers"
                fi
                ;;

            "TEST")
                printf "$(logtime) $GRAY$BOLD T $END $HOSTNAME %-15s %-4s ${*:2}\n" $(baseName) $BASH_LINENO >> "$logdir"
                ;;

            "CODE")
                printf "%-19s $RED$BOLD E $END $HOSTNAME codesys              ${*:3}\n" ${2:0:19}  >> "$logdir"
                ;;

            "FW")
                printf "$(logtime) $BLUE$BOLD F $END $HOSTNAME firmwareupdate       from %s to %s ${*:4}\n" $2 $3  >> "$logdir"
                ;;

            "FIRST")
                printf "$(logtime) $BLUE$BOLD F $END $HOSTNAME firstboot            FW version %s ${*:3}\n" $2  >> "$logdir"
                ;;

            "-h" | "--help")
                printf "\n" &&
                printf "'-------------------------------------------------'\n" &&
                printf "plcLog %s %s %s\n" '$1' '$2' '$3' &&
                printf "%s  INFO, ERROR, WARN, WEB, SSH, CODE, FW \n" '$1'  &&
                printf "      └─> --newbase\n"  &&
                printf "              └─> <new base name>\n" 
                ;;

            *)
                printf "$(logtime) $RED$BOLD N $END $HOSTNAME %-15s %-4s not found! ${*:2}\n" $(baseName) $BASH_LINENO >> "$logdir"

    esac
}


#-------------------------------------------------------
# Direct log
#-------------------------------------------------------
if [ "$1" == '--lock' ]; 
then 
    if [ "$2" == '-ok' ]; then
        printf "$(logtime) $BLUE$BOLD F $END $HOSTNAME firmwarelock         force dpkg to reinstall firmware package\n"  >> "$logdir"
    elif [ "$2" == '-nofw' ]; then
        printf "$(logtime) $RED$BOLD F $END $HOSTNAME firmwarelock         update.lock was there, but the firmware was not\n"  >> "$logdir"
    fi
fi
