#!/bin/bash

#-------------------------------------------------------------
# INSTALL EXTRA PACKAGES
# 
# September/2023
#
# List of HW
# PLC500   = 7
# PLC500ED = 3
# PLC500MC = 6
# PLC410   = 0
#
#-------------------------------------------------------------


#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh


#-------------------------------------------------------------
# Debug echo Function
#-------------------------------------------------------------
function debug_echo {
    if [[ -n "$DEBUG_FLAG" ]]; then           # Exibe a mensagem apenas se DEBUG_FLAG estiver definida
        echo "DEBUG_$OPTLOG: $1" | tee /dev/kmsg;   
    fi
}


#-------------------------------------------------------------
# Remove package Function
#-------------------------------------------------------------
function rm_package {

    debug_echo "entrou rm_package"

    local PACKAGE_FILE="$1"
    local PACKAGE_NAME=$(dpkg-deb -f "$1" Package);

    if [[ -n "$PACKAGE_FILE" ]] && [[ -n "$PACKAGE_NAME" ]]; 
    then 
        if dpkg -l | grep -qw "$PACKAGE_NAME" && [[ -f "$PACKAGE_FILE" ]]; then
            rm "$PACKAGE_FILE";
        fi
    else 
        result=1; 
    fi
}


#-------------------------------------------------------------
# Wait dpkg Function
#-------------------------------------------------------------
function wait_dpkg {
    local dpkgTime=10

    # Verify dpkg process running
    i=0; while pgrep -x dpkg >/dev/null || fuser /var/lib/dpkg/lock* >/dev/null; do sleep 1; ((i++)); if [[ "$i" -ge "$dpkgTime" ]]; then break; fi; done
}


function installDEB {

    debug_echo "entrou installDEB $1"

    local PACKAGE_FILE="$1"
    local INSTALL_FLAG=0
    local PACKAGE_VERS_INSTALLED      

    # Verify argumens and extract info about packages files
    if [[ -z "$1" ]]; 
    then 
        debug_echo "first argument does not exist"; 
        result=1;
    else
        if [[ -f "$PACKAGE_FILE" ]]; 
        then
            PACKAGE_NAME=$(dpkg-deb -f "$1" Package);
            PACKAGE_VERS=$(dpkg-deb -f "$1" Version); 
            debug_echo "PACKAGE: $PACKAGE_NAME, VERSION: $PACKAGE_VERS"; 
        else
            debug_echo "return = 1"; 
            return 1;
        fi
    fi


    # Verify installed package version
    if dpkg -l | grep -qw "$PACKAGE_NAME";
    then
        PACKAGE_VERS_INSTALLED=$(dpkg-query -W -f='${Version}' "$PACKAGE_NAME")
        if dpkg --compare-versions "$PACKAGE_VERS_INSTALLED" "lt" "$PACKAGE_VERS"; then INSTALL_FLAG=1; fi
    fi


    # Wait for dpkg to finish
    wait_dpkg
   
    # Install function
    if ! dpkg -l | grep -qw "$PACKAGE_NAME" || [[ "$INSTALL_FLAG" == 1 ]];
    then
        if [[ -f "$PACKAGE_FILE" ]] 
        then 
            dpkg --force-all -i "$PACKAGE_FILE"; RES1=$?

            if [[ "$RES1" -eq 0 ]]; 
            then
                if dpkg -l | grep -qw "$PACKAGE_NAME";
                then
                    plcLog INFO package "$PACKAGE_NAME" "$PACKAGE_VERS" was installed
                else
                    plcLog ERROR package "$PACKAGE_NAME" could not be installed
                    result=1;
                fi
            else
                plcLog ERROR package "$PACKAGE_NAME" could not be installed
                result=1;
            fi
        else 
            plcLog ERROR package "$PACKAGE_NAME" do not exist and it is not installed
        fi
    fi
}


#-------------------------------------------------------------
# Uninstall Packages Function
#-------------------------------------------------------------
function uninstallDEB {

    debug_echo "entrou uninstallDEB $1"

    # Verify argumens 
    if [[ -z "$1" ]]; 
    then 
        debug_echo "first argument does not exist"; 
        result=1;
    fi

    if dpkg -l | grep -qw "$1"; then n=0; 
        until [[ "$n" -ge 5 ]]; do
            if dpkg -P "$1"; 
            then 
                plcLog INFO package "$1" was uninstalled; break;
            else
                n=$((n+1));
            fi

            if [[ $n -ge 5 ]]; then plcLog ERROR package "$1" could not be uninstalled; result=1; fi
        done; 
    fi

}


#-------------------------------------------------------------
# INITIAL CONDITIONS
#-------------------------------------------------------------
#DEBUG_FLAG="comentar_para_nao_existir_logs"
OPTLOG="PLCINSTALL"
result=0

#-------------------------------------------------------------
# DEBIAN FILES
#-------------------------------------------------------------
VSFTPD=/opt/weg/packages/vsftpd_3.0.3-12_armhf.deb
BRIDGE_UTILS=/opt/weg/packages/bridge-utils_1.6-2_armhf.deb
CODEMETER=/opt/weg/packages/codemeter-lite_8.10.6230.501_armhf.deb
FUSE=/opt/weg/packages/fuse_2.9.9-5_armhf.deb
LIBFUSE2=/opt/weg/packages/libfuse2_2.9.9-5_armhf.deb
EXFATFUSE=/opt/weg/packages/exfat-fuse_1.3.0-1_armhf.deb
EXFATUTILS=/opt/weg/packages/exfat-utils_1.3.0-1_armhf.deb

# iptables related
LIBNFTNL11=/opt/weg/packages/libnftnl11_1.1.2-2_armhf.deb
LIBNFTABLES0=/opt/weg/packages/libnftables0_0.9.0-2_armhf.deb
NFTABLES=/opt/weg/packages/nftables_0.9.0-2_armhf.deb
LIBIP6TC0=/opt/weg/packages/libip6tc0_1.8.2-4_armhf.deb
LIBIPTC0=/opt/weg/packages/libiptc0_1.8.2-4_armhf.deb
LIBNFNETLINK0=/opt/weg/packages/libnfnetlink0_1.0.1-3+b1_armhf.deb
LIBNETFILTER_CONNTRACK3=/opt/weg/packages/libnetfilter-conntrack3_1.0.7-1_armhf.deb
IPTABLES=/opt/weg/packages/iptables_1.8.2-4_armhf.deb


#-------------------------------------------------------------
# Main Program
#-------------------------------------------------------------
debug_echo "Starting plcInstall.sh..."

mkdir -p /opt/weg/packages

uninstallDEB dmidecode

installDEB "$VSFTPD"                  && rm_package "$VSFTPD"
installDEB "$LIBFUSE2"                && rm_package "$LIBFUSE2"
installDEB "$FUSE"                    && rm_package "$FUSE"
installDEB "$EXFATUTILS"              && rm_package "$EXFATUTILS"
installDEB "$EXFATFUSE"               && rm_package "$EXFATFUSE"
installDEB "$BRIDGE_UTILS"            && rm_package "$BRIDGE_UTILS"
installDEB "$CODEMETER"               && rm_package "$CODEMETER"
installDEB "$LIBNFTNL11"              && rm_package "$LIBNFTNL11"
installDEB "$LIBNFTABLES0"            && rm_package "$LIBNFTABLES0"
installDEB "$NFTABLES"                && rm_package "$NFTABLES"
installDEB "$LIBIP6TC0"               && rm_package "$LIBIP6TC0"
installDEB "$LIBIPTC0"                && rm_package "$LIBIPTC0"
installDEB "$LIBNFNETLINK0"           && rm_package "$LIBNFNETLINK0"
installDEB "$LIBNETFILTER_CONNTRACK3" && rm_package "$LIBNETFILTER_CONNTRACK3"
installDEB "$IPTABLES"                && rm_package "$IPTABLES"


#-------------------------------------------------------------
# Verify dpkg process running
#-------------------------------------------------------------
wait_dpkg


#-------------------------------------------------------------
# Post install
#-------------------------------------------------------------
systemctl disable codemeter
systemctl disable codemeter-webadmin


#-------------------------------------------------------------
# Result of the installations 
#-------------------------------------------------------------
debug_echo "Finished plcInstall.sh with result = $result"
exit "$result"
