#!/bin/bash

#-------------------------------------------------------------
# LOG ROTATION
#
# Julho/2023
# 
# - Define o tamanho máximo do arquivo de log
# - Define o número de arquivos de log
# - Renomeia os logs antigos
# - Exclui arquivos antigos acima do limite previsto
#
# script localizado em: /opt/weg/scripts/ 
#-------------------------------------------------------------


#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh


#-------------------------------------------------------
# Verifica o hardware
#-------------------------------------------------------
HW_VERSION=$(grep -oP '(?<="HW_VERSION": )[0-9]+' /etc/weg/firstBoot.json) 


#-------------------------------------------------------
# Se PLC500ED, restart logrotate
#-------------------------------------------------------
if [[ "$HW_VERSION" == "3" ]]; then systemctl restart logrotate; fi


#-------------------------------------------------------------
# Declaração de variáveis
#-------------------------------------------------------------
log_file="/var/log/weg/plcLogs.log"
max_size=10485760  # 10 MB (in bytes)
backup_count=2     # Number of backup files to keep

# Check if the log file exists
if [ ! -f "$log_file" ]; then
    exit 1
fi

# Check if the log file exceeds the maximum size
if [ $(stat -c %s "$log_file") -ge $max_size ]; then

    # Log
    plcLog INFO PLC Log file reached 10 MB. Starting rotation..

    # Rotate the log file
    for ((i=$backup_count; i>0; i--)); do
        if [ -f "$log_file.$i" ]; 
        then
            mv "$log_file.$i" "$log_file.$((i+1))"
        fi
    done

    mv "$log_file" "$log_file.1"
    touch "$log_file"
fi

# Remove older log files
if [ -f "${log_file}.4" ]; then
    rm "${log_file}.4"
fi

if [ -f "${log_file}.5" ]; then
    rm "${log_file}.5"
fi

if [ -f "${log_file}.6" ]; then
    rm "${log_file}.6"
fi

if [ -f "${log_file}.7" ]; then
    rm "${log_file}.7"
fi


#-------------------------------------------------------------
# Remove logs com mais de 50 MB (apenas na pasta /var/log)
#-------------------------------------------------------------
find /var/log -maxdepth 1 -type f -size +50M -exec rm {} \;