#!/bin/bash

#-------------------------------------------------------------
# PLC UPDATE SCRIPT
# September, 2023.
# 
# - Move plctools to /usr/bin
# - Execute plctools -vgpg
# - Execute plctools -u
# - Execute plcinit.service
#
#-------------------------------------------------------------

#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh


#-------------------------------------------------------------
# If exists, move plctools to /usr/bin
#-------------------------------------------------------------
if [[ -f "/opt/weg/scripts/plctools" ]]; 
then cd /opt/weg/scripts/ && mv plctools /usr/bin/
fi


#-------------------------------------------------------------
# If exists, move plcmedia to /usr/bin
#-------------------------------------------------------------
if [[ -f "/opt/weg/scripts/plcmedia" ]]; 
then cd /opt/weg/scripts/ && mv plcmedia /usr/bin/
fi


#-------------------------------------------------------------
# Enable GPIO66 (PINO 7 - Last 4 DOs and SPI)
#-------------------------------------------------------------
[ ! -d "/sys/class/gpio/gpio66" ] && echo 66 > /sys/class/gpio/export
[ -d "/sys/class/gpio/gpio66" ]   && echo out > /sys/class/gpio/gpio66/direction
[ -d "/sys/class/gpio/gpio66" ]   && echo 1 > /sys/class/gpio/gpio66/value


#-------------------------------------------------------------
# Loading LEDs
#-------------------------------------------------------------
/opt/weg/scripts/plcleds.sh --loading &


#-------------------------------------------------------------
# First log to the plcxWegLib.log
#-------------------------------------------------------------
chmod +x /usr/bin/plctools
plctools --log -first


#-------------------------------------------------------------
# Firmware verify and update
#-------------------------------------------------------------
plctools -vgpg
returnVerify=$?

# Se retorno da verificação = 0
if [[ "$returnVerify" == "0" ]];
then

    # Update de firmware
    plctools -u
    returnUpdate=$?

    # Se retorno do update != 0
    if [[ "$returnUpdate" != "0" ]]; then plcLog ERROR plctools -u returned "$returnUpdate"; fi

else

    # Start plcinit
    systemctl start plcinit &

fi

