#!/bin/bash

#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh

#-------------------------------------------------------------
# Trap to catch line errors
#-------------------------------------------------------------
trap 'plcLog ERROR Trap!' ERR

#---------------------------------------------------------
# Verifica se existe o dispositivo
#---------------------------------------------------------
if ls /dev/mmcblk0p1 1> /dev/null 2>&1;
then

    #---------------------------------------------------------
    # Verify if the filesystem format is vfat or exfat
    #---------------------------------------------------------
    if blkid -s TYPE /dev/mmcblk0p1 | grep -q -E 'exfat|vfat'; 
    then

        #---------------------------------------------------------
        # Verifica se não existe pasta /mnt/sdcard. Cria se não existir.
        #---------------------------------------------------------
        n1=0
        if [[ ! -d "/mnt/sdcard" ]]; 
        then 
            until [[ "$n1" -ge 20 ]]
            do      
                if [[ ! -d "/mnt/sdcard" ]]; 
                then 
                    mkdir -p /mnt/sdcard; 
                    n1=$((n1+1)); 
                    sleep 1;
                    if [[ "$n1" -ge 20 ]]; then plcLog WARN SD card folder could NOT be created; fi
                else 
                    n1=100; 
                fi
            done
        fi
    
        #---------------------------------------------------------
        # Read /etc/weg/ftp/ftpcfg.json and create symbolic link
        #---------------------------------------------------------
        if [[ -f "/etc/weg/ftp/ftpcfg.json" ]]; then

            ftpcfg=$(cat /etc/weg/ftp/ftpcfg.json)

            FTP_SD=$(grep -oP '"FTP_SD": \K\d+' <<< "$ftpcfg")

            if [[ "$FTP_SD" == "1" ]]; then 
                if [[ -d "/mnt/sdcard" ]]; then
                    mkdir -p /home/wegftp/ftp/files/SDcard
                    mkdir -p /var/lib/weg/codesys/ftp/SDcard
                    mount --bind /mnt/sdcard /home/wegftp/ftp/files/SDcard
                    mount --bind /mnt/sdcard /var/lib/weg/codesys/ftp/SDcard
                fi
            fi
        fi

        #---------------------------------------------------------
        # Confere se está montado. Se não estiver, tenta novamente
        #---------------------------------------------------------
        n1=0
        until [[ "$n1" -ge 20 ]]
        do  
            if mount | grep -q mmcblk0p1 > /dev/null; 
            then 
                n1=100;
            else
                if blkid -s TYPE /dev/mmcblk0p1 | grep -q 'vfat';
                    then mount -o uid=wegftp,gid=wegftp,flush,nofail /dev/mmcblk0p1 /mnt/sdcard && plcLog INFO SD card mounted as VFAT;
                elif blkid -s TYPE /dev/mmcblk0p1 | grep -q 'exfat';
                    then mount -t exfat -o uid=wegftp,gid=wegftp,flush,nofail /dev/mmcblk0p1 /mnt/sdcard && plcLog INFO SD card mounted as EXFAT;
                fi

                if mount | grep mmcblk0p1 > /dev/null; 
                then n1=100; 
                else n1=$((n1+1)); sleep 1; if [ "$n1" -ge 20 ]; then plcLog WARN SD card could NOT be mounted; fi
                fi

                if ! ls /dev/mmcblk0p1 1> /dev/null 2>&1;
                then n1=100; plcLog WARN SD card was removed before mount;
                fi
            fi
        done

        #---------------------------------------------------------
        # Read /etc/weg/ftp/ftpcfg.json and create symbolic link
        #---------------------------------------------------------
        if [[ -f "/etc/weg/ftp/ftpcfg.json" ]]; then

            ftpcfg=$(cat /etc/weg/ftp/ftpcfg.json)

            FTP_SD=$(grep -oP '"FTP_SD": \K\d+' <<< "$ftpcfg")

            if [[ "$FTP_SD" == "1" ]]; then 
                if [[ -d "/mnt/sdcard" ]]; then
                    if [[ ! -L "/var/lib/weg/codesys/ftp/SDcard" ]]; then
                        ln -s "/mnt/sdcard" "/var/lib/weg/codesys/ftp/SDcard"
                    fi
                fi
            fi

        fi


    else
        plcLog WARN SD card in wrong format;

    fi

    #---------------------------------------------------------
    # Se está montado corretamente, executa o plcmedia
    #---------------------------------------------------------
    if mount | grep -qw 'mmcblk0p1' && ! pgrep -f "plcmedia" > /dev/null; then
        nohup plcmedia --sdcard > /dev/null 2>&1 &
    fi

else
    if mount | grep mmcblk0p1 > /dev/null;  then /usr/bin/umount -f -l "/mnt/sdcard"; fi
    if [[ -d "/mnt/sdcard" ]]; then rm -r /mnt/sdcard; fi

fi