#!/bin/bash

#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh


#---------------------------------------------------------
# Verifica se existe o dispositivo
#---------------------------------------------------------
if ls /dev/sd* 1> /dev/null 2>&1;
then

    #---------------------------------------------------------
    # Verify if the filesystem format is vfat or exfat
    #---------------------------------------------------------
    if blkid -s TYPE /dev/sd* | grep -q -E 'exfat|vfat'; 
    then

        #---------------------------------------------------------
        # Verifica se não existe pasta /mnt/usb. Cria se não existir.
        #---------------------------------------------------------
        n1=0
        if [[ ! -d "/mnt/usb" ]];
        then 
            until [[ "$n1" -ge 20 ]]
            do      
                if [[ ! -d "/mnt/usb" ]]; 
                then 
                    mkdir -p /mnt/usb; 
                    n1=$((n1+1)); 
                    sleep 1;
                    if [[ "$n1" -ge 20 ]]; then plcLog WARN USB folder could NOT be created; fi
                else 
                    n1=100; 
                fi
            done
        fi

        #---------------------------------------------------------
        # Read /etc/weg/ftp/ftpcfg.json and create symbolic link
        #---------------------------------------------------------
        if [[ -f "/etc/weg/ftp/ftpcfg.json" ]]; then

            ftpcfg=$(cat /etc/weg/ftp/ftpcfg.json)

            FTP_USB=$(grep -oP '"FTP_USB": \K\d+' <<< "$ftpcfg")

            if [[ "$FTP_USB" == "1" ]]; then 
                if [[ -d "/mnt/usb" ]]; then
                    mkdir -p /home/wegftp/ftp/files/USB
                    mkdir -p /var/lib/weg/codesys/ftp/USB
                    mount --bind /mnt/usb /home/wegftp/ftp/files/USB
                    mount --bind /mnt/usb /var/lib/weg/codesys/ftp/USB
                fi
            fi

        fi

        #---------------------------------------------------------
        # Confere se está montado. Se não estiver, tenta novamente
        #---------------------------------------------------------
        n1=0
        until [[ "$n1" -ge 20 ]]
        do      
            MEDIA=$( ls /dev | grep sd.1 )
            if [ -z "$MEDIA" ]; 
            then 
                MEDIA=$( ls /dev | grep sd. ) 
            else
                MEDIA=$( ls /dev | grep sd.1 )
            fi

            if grep -qs '/dev/sd.' "/proc/mounts"; 
            then 
                n1=100;
            else
                if blkid -s TYPE /dev/sd* | grep -q 'vfat';
                    then mount -o uid=wegftp,gid=wegftp,flush,nofail /dev/$MEDIA "/mnt/usb" && plcLog INFO USB device mounted as VFAT;
                elif blkid -s TYPE /dev/sd* | grep -q 'exfat';
                    then mount -t exfat -o uid=wegftp,gid=wegftp,flush,nofail /dev/$MEDIA "/mnt/usb" && plcLog INFO USB device mounted as EXFAT;
                fi

                if grep -qs '/dev/sd.' "/proc/mounts"; 
                then n1=100; 
                else n1=$((n1+1)); sleep 1; if [[ "$n1" -ge 20 ]]; then plcLog WARN USB could NOT be mounted; fi
                fi

                if ! ls /dev/sd* 1> /dev/null 2>&1;
                then n1=100; plcLog WARN USB was removed before mount;
                fi
            fi
        done

        #---------------------------------------------------------
        # Read /etc/weg/ftp/ftpcfg.json and create symbolic link
        #---------------------------------------------------------
        if [[ -f "/etc/weg/ftp/ftpcfg.json" ]]; then

            ftpcfg=$(cat /etc/weg/ftp/ftpcfg.json)

            FTP_USB=$(grep -oP '"FTP_USB": \K\d+' <<< "$ftpcfg")

            if [[ "$FTP_USB" == "1" ]]; then 
                if [[ -d "/mnt/USB" ]]; then
                    ln -s "/mnt/USB" "/var/lib/weg/codesys/ftp/USB"
                fi
            fi

        fi

    else
        plcLog WARN USB in wrong format;
    
    fi

    #---------------------------------------------------------
    # Se está montado corretamente, executa o plcmedia
    #---------------------------------------------------------
    if grep -qs '/dev/sd.' /proc/mounts && ! pgrep -f "plcmedia" > /dev/null; then
        nohup plcmedia --usb > /dev/null 2>&1 &
    fi

else
    if grep -qs '/dev/sd.' "/proc/mounts"; then /usr/bin/umount -f -l /mnt/usb; fi
    if [[ -d "/mnt/usb" ]]; then rm -r /mnt/usb; fi

fi