#!/bin/bash

#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh

#-------------------------------------------------------------
# Trap to catch line errors
#-------------------------------------------------------------
trap 'plcLog ERROR Trap!' ERR

mode=$1
brIP=$2
brmask=$3
brAssigment=$4

inet1="eth1"
inet1ip="192.168.1.10"
inet1mask="255.255.255.0"

inet2="eth2"
inet2ip="192.168.2.10"
inet2mask="255.255.255.0"

br="br1"

TOML_FILE="/etc/core-app/config.toml"

#-------------------------------------------------------------
# Get the main variable
#-------------------------------------------------------------
HW_VERSION=$(grep -oP '(?<="HW_VERSION": )[0-9]+' /etc/weg/firstBoot.json)

#-------------------------------------------------------------
# Função para adicionar ou substituir o bloco interfaces.br1
#-------------------------------------------------------------
add_interfaces_br1() {
    # Temporário para o novo conteúdo do arquivo
    TEMP_FILE=$(mktemp)

    if [[ "$brAssigment" == "dynamic" ]]; then 
        dhcp=true
        brIP="192.168.0.10"
        brmask="255.255.255.0"
    else 
        # Configuring bridge IP
        dhcp=false
    fi

    # Indica se o bloco foi encontrado
    block_found=false
    block_added=false

    # Lê o arquivo linha por linha
    while IFS= read -r line || [[ -n "$line" ]]; do
        # Verifica se encontrou o início do bloco interfaces.br1
        if [[ $line =~ ^\[interfaces\.br1\] ]]; then
            block_found=true
            # Escreve o novo bloco no arquivo temporário
            if ! $block_added; then
                echo "" >> "$TEMP_FILE"
                cat <<EOL >>"$TEMP_FILE"
[interfaces.br1]
active = true
dhcp = $dhcp
default = false
ip = "$brIP"
netmask = "$brmask"
EOL
                block_added=true
            fi
        # Verifica se encontrou o início de um novo bloco
        elif [[ $line =~ ^\[.*\] ]] && $block_found; then
            block_found=false
            if ! $block_added; then
                echo "" >> "$TEMP_FILE"
                cat <<EOL >>"$TEMP_FILE"
[interfaces.br1]
active = true
dhcp = $dhcp
default = false
ip = "$brIP"
netmask = "$brmask"
EOL
                block_added=true
            fi
            echo "" >> "$TEMP_FILE"
            echo "$line" >> "$TEMP_FILE"
        elif ! $block_found; then
            echo "$line" >> "$TEMP_FILE"
        fi
    done < "$TOML_FILE"

    # Se o bloco não foi encontrado, adiciona no final
    if ! grep -q "^\[interfaces\.br1\]" "$TEMP_FILE"; then
        echo "" >> "$TEMP_FILE"
        cat <<EOL >>"$TEMP_FILE"

[interfaces.br1]
active = true
dhcp = $dhcp
default = false
ip = "$brIP"
netmask = "$brmask"
EOL
    fi

    # Remove linhas em branco duplicadas
    awk 'BEGIN{blank=0} /^$/ {blank++; if (blank==1) print $0} !/^$/ {blank=0; print $0}' "$TEMP_FILE" > "${TEMP_FILE}.tmp"
    mv "${TEMP_FILE}.tmp" "$TEMP_FILE"

    # Substitui o arquivo original pelo novo
    mv "$TEMP_FILE" "$TOML_FILE"
}

#-------------------------------------------------------------
# Função para remover o bloco interfaces.br1
#-------------------------------------------------------------
remove_interfaces_br1() {
    # Temporário para o novo conteúdo do arquivo
    TEMP_FILE=$(mktemp)

    # Indica se o bloco foi encontrado
    block_found=false

    # Lê o arquivo linha por linha
    while IFS= read -r line || [[ -n "$line" ]]; do
        # Verifica se encontrou o início do bloco interfaces.br1
        if [[ $line =~ ^\[interfaces\.br1\] ]]; then
            block_found=true
        # Verifica se encontrou o início de um novo bloco
        elif [[ $line =~ ^\[.*\] ]] && $block_found; then
            block_found=false
            echo "" >> "$TEMP_FILE"
            echo "$line" >> "$TEMP_FILE"
        elif ! $block_found; then
            echo "$line" >> "$TEMP_FILE"
        fi
    done < "$TOML_FILE"

    # Remove linhas em branco duplicadas
    awk 'BEGIN{blank=0} /^$/ {blank++; if (blank==1) print $0} !/^$/ {blank=0; print $0}' "$TEMP_FILE" > "${TEMP_FILE}.tmp"
    mv "${TEMP_FILE}.tmp" "$TEMP_FILE"

    # Substitui o arquivo original pelo novo
    mv "$TEMP_FILE" "$TOML_FILE"
}


if [ "$mode" == "--add" ]; then
    
    # If don't exist create it
    if ! ip link show "$br" &> /dev/null; then
        # Remove IP from interfaces
        ip addr flush dev "$inet1"; ip addr flush dev "$inet2"
        
        # Create a new bridge interface
        brctl addbr "$br"

        # network interfaces we want to bridge
        brctl addif "$br" "$inet1" "$inet2"

        # Bring up bridge interface
        ifconfig "$br" up
    fi

    #-------------------------------------------------------------
    # Opções para o PLC500ED
    #-------------------------------------------------------------
    if [[ "$HW_VERSION" == "3" ]]; then 
        add_interfaces_br1
        iptables -A FORWARD -i br1 -o br1 -j ACCEPT
    fi

    if [[ "$brAssigment" == "dynamic" ]]; then 
        # Configuring as dinamic
        ip addr flush dev eth1
        ip addr flush dev eth2
        ip addr flush dev br1

        # Get IP
        dhclient br1
    else 
        # Configuring bridge IP
        ifconfig "$br" "$brIP" netmask "$brmask"
    fi

    # So there are two options either we add Iptables rule to allow these packets or completely disable Netfilter calling Iptables. 
    #sysctl net.bridge.bridge-nf-call-iptables
    #sysctl -w net.bridge.bridge-nf-call-iptables=0


elif [ "$mode" == "--rm" ]; then

    # Remove IP from bridge
    ip addr flush dev eth1
    ip addr flush dev eth2
    ip addr flush dev "$br"
    ifconfig "$br" down
    brctl delbr "$br"

    # Configuring eths
    ifconfig "$inet1" "$inet1ip" netmask "$inet1mask"
    ifconfig "$inet2" "$inet2ip" netmask "$inet2mask"

    #-------------------------------------------------------------
    # Opções para o PLC500ED
    #-------------------------------------------------------------
    if [[ "$HW_VERSION" == "3" ]]; then 
        remove_interfaces_br1
        iptables -D FORWARD -i br1 -o br1 -j ACCEPT
    fi

else
    echo "WRONG MODE"
fi


# Ver interfaces em uma bridge
# brctl show br0


# BOND
# bonding module
# modprobe bonding
# ifconfig eth1 down; ifconfig eth2 down;
# ip link add bond0 type bond mode balance-rr
# ip link set eth1 master bond0
# ip link set eth2 master bond0
# ifconfig bond0 192.168.0.10 netmask 255.255.255.0; ifconfig bond0 up
