#!/bin/bash


#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh


#-------------------------------------------------------------
# Trap to catch line errors
#-------------------------------------------------------------
trap 'plcLog ERROR Trap!' ERR


while :
do

    sleep 5

    #-------------------------------------------------------------
    # Listar processos, extrair o PID, a porcentagem de CPU e o nome do processo
    #-------------------------------------------------------------
    process_info=$(ps -eo pid,%cpu,comm)


    #-------------------------------------------------------------
    # Ordena a saída pelo uso de CPU em ordem decrescente e limita aos três primeiros processos
    #-------------------------------------------------------------
    # O comando 'tail -n +2' remove o cabeçalho da saída.
    # O comando 'head -n 4' limita a saída aos três primeiros processos (cabeçalho excluído).

    top_processes=$(echo "$process_info" | sort -k2 -nr | head -n 4)


    #-------------------------------------------------------------
    # Itera sobre o resultado para acessar cada posição de forma independente
    #-------------------------------------------------------------
    #echo "Os três processos que estão consumindo mais CPU são:"
    while read -r line; do
        # Use 'awk' para dividir a linha em campos usando espaço em branco como delimitador.
        pid=$(echo "$line" | awk '{print $1}')
        cpu_percent=$(echo "$line" | awk '{print int($2)}')  # Converta para inteiro
        process_name=$(echo "$line" | awk '{ $1=$2=""; sub(/^[ \t]+/, "", $0); print }')

        # Verifique se a porcentagem de CPU é maior que 8
        if [ "$cpu_percent" -gt 8 ] && [ "$process_name" != "codesyscontrol" ]; then
            plcLog ERROR $process_name está com CPU maior que 8 porc
        fi
        
        # Imprima as informações de cada processo.
        #echo "PID: $pid, CPU: $cpu_percent%, Processo: $process_name"
    done <<< "$top_processes"


done