#!/bin/bash

#-------------------------------------------------------------
# SCRIPT PARA A CONFIGURACAO APÓS O PRIMEIRO BOOT 
# 
# Novembro/2023
# 
# - Printa no log que o script iniciou
# - Verifica o HW: 
#     - se for a giga de gravação, apenas pisca os leds verdes forever
#     - Se for um HW comercial:
#           - Executa o plctools --init para a configuração do fw de acordo
#           com o hardware
#           - Executa o plcd
#           - Desmascara e habilita o plcUpdate, plcinit e weg-coreapp (PLC500ED)  
#-------------------------------------------------------------


#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh


#-------------------------------------------------------------
# Trap to catch line errors
#-------------------------------------------------------------
trap 'plcLog ERROR Trap!' ERR


#-------------------------------------------------------
# Log
#-------------------------------------------------------
plcLog INFO script plchwconfig inicializado


#-------------------------------------------------------------
# Configurando e setando GPIO66 (PINO 7 - Enable 4 ultimas saidas e LEDSs SPI)
#-------------------------------------------------------------
[[ ! -d "/sys/class/gpio/gpio66" ]] && echo 66 > /sys/class/gpio/export
[[ -d "/sys/class/gpio/gpio66" ]]   && echo out > /sys/class/gpio/gpio66/direction
[[ -d "/sys/class/gpio/gpio66" ]]   && echo 1 > /sys/class/gpio/gpio66/value


#-------------------------------------------------------
# Verifica o hardware
#-------------------------------------------------------
(plctools --hw_read && HW_VERSION=$?) || { HW_VERSION=$?; true; }


# Verifica se é o firmware PLC500ED e outro HW
[[ -f "/var/lib/edge_device" ]] && [[ "$HW_VERSION" != "3" ]] && ED_HW_FW=1;


# Verifica se é o HW do PLC500ED e outro FW
[[ ! -f "/var/lib/edge_device" ]] && [[ "$HW_VERSION" == "3" ]] && ED_HW_FW=2;


if [[ "$ED_HW_FW" == "1" ]];
then
    /opt/weg/scripts/plcleds.sh --invalidEDFW &
    plcLog ERROR firmware do PLC500ED gravado em outro hardware
    sync
    plcd &


elif [[ "$ED_HW_FW" == "2" ]];
then
    /opt/weg/scripts/plcleds.sh --invalidEDHW &
    plcLog ERROR hardware do PLC500ED gravado com outro firmware
    sync
    plcd &


elif [[ "$HW_VERSION" != "2" ]];
then 

    #-------------------------------------------------------
    # Ajuste da data e hora do hardware e do sistema
    #-------------------------------------------------------
    hwclock --set --date "01/01/2024 00:00:00"
    hwclock --hctosys


    #-------------------------------------------------------------
    # Executa o daemon plctools --init
    #-------------------------------------------------------------
    if [[ -e "/usr/bin/plctools" ]]; 
    then 
        cd /usr/bin/; chmod +x plctools; plctools --init
        returnPlctools=$?

        if [[ "$returnPlctools" != "0" ]]; 
        then 
            /opt/weg/scripts/plcleds.sh --configfail & 
            plcd &
        fi

    else 
        plcLog ERROR there was no plctools inside /usr/bin
    fi


    #-------------------------------------------------------
    # Desmascara e habilita o serviço da webpage ou coreapp
    #-------------------------------------------------------
    if [[ "$HW_VERSION" == "3" ]]; 
    then 
        systemctl unmask weg-coreapp
        systemctl enable weg-coreapp
    fi


    #-------------------------------------------------------
    # Deixa mensagem no log
    #-------------------------------------------------------
    plcLog INFO script plchwconfig finalizado


    #-------------------------------------------------------
    # Desmascara e habilita o serviço do plcUpdate
    #-------------------------------------------------------
    systemctl unmask plcUpdate
    systemctl enable plcUpdate


    #-------------------------------------------------------
    # Desabilita o serviço de plchwconfig
    #-------------------------------------------------------
    systemctl disable plchwconfig
    systemctl daemon-reload
    
    sync


    #-------------------------------------------------------
    # Reinicia o produto
    #-------------------------------------------------------
    reboot


else
    #-------------------------------------------------------
    # Sync all cached file data of the current user
    #-------------------------------------------------------
    sync


    #-------------------------------------------------------
    # Executa plcleds.sh --giga e aguarda reboot
    #-------------------------------------------------------
    /opt/weg/scripts/plcleds.sh --giga &
    plcLog WARN PLC energizado na GIGA de gravação


    #-------------------------------------------------------------
    # Executa o daemon plcd
    #-------------------------------------------------------------
    plcd &

fi