#!/bin/bash

#-------------------------------------------------------------
# LEDS DE STATUS
# September/2023
# 
# script localizado em: /opt/weg/scripts/
#-------------------------------------------------------------

LED1g="\x01"  # 000001
LED1r="\x02"  # 000010
LED1o="\x03"  # 000011

LED2g="\x04"  # 000100
LED2r="\x08"  # 001000
LED2o="\x0C"  # 001100

LED3g="\x10"  # 010000
LED3r="\x20"  # 100000
LED3o="\x30"  # 110000

LED4o="\x40"    # 1000000
LED43o="\x70"   # 1110000
LED432o="\x7C"  # 1111100
LED4321o="\x7F" # 1111111

LED4r="\x80"    # 10 00 00 00
LED41r="\x82"   # 10 00 00 10
LED42r="\x88"   # 10 00 10 00
LED43r="\xA0"   # 10 10 00 00
LED431r="\xA2"  # 10 10 00 10
LED432r="\xA8"  # 10 10 10 00
LED4321r="\xAA" # 10 10 10 10


LED123r4o="\xEA" # 11 10 10 10
LED124r3o="\xBA" # 10 11 10 10
LED134r2o="\xAE" # 10 10 11 10
LED234r1o="\xAB" # 10 10 10 11

LED41o23r="\xEB" # 11 10 10 11
LED42o13r="\xEE" # 11 10 11 10
LED43o12r="\xFA" # 11 11 10 10
LED431o2r="\xFB" # 11 11 10 11
LED432o1r="\xFE" # 11 11 11 10


LED12g="\x05"  # 000101
LED12r="\x0A"  # 001010
LED12o="\x0F"  # 001111

LED23g="\x14"  # 010100
LED23r="\x28"  # 101000
LED23o="\x3C"  # 111100

LED123o4r="\xBF" # 10 11 11 11
LED124o3r="\xEF" # 11 10 11 11
LED134o2r="\xFB" # 11 11 10 11
LED234o1r="\xFE" # 11 11 11 10

LED4321o="\xFF"  # 11 11 11 11
LED124o3g="\xDF" # 11 01 11 11
LED134o2g="\xF7" # 11 11 01 11
LED234o1g="\xFD" # 11 11 11 01

LED42o="\xCC" # 11 00 11 00
LED41o="\xC3" # 11 00 00 11
LED431o="\xF3" # 11 11 00 11

LED123g="\x15"  # 010101

LEDNULL="\x00"  # 000000
LEDg="\x15"  # 010101
LEDo="\x3F"  # 111111
LEDr="\x2A"  # 101010

LEDedge="\xD7" # 11 01 01 11

SPIDEVPATH="/dev/spidev3.0" 
EDGE_DEVICE="/var/lib/edge_device"
FOLDER="/opt/core-app" 
FLAGS_TMP_DIR="/tmp/flags"
FLAGS_FLASH_DIR="/var/lib/weg/flags"
LOCKFILE="${FLAGS_TMP_DIR}/plcleds.lock"
GIGAFILE="${FLAGS_FLASH_DIR}/plcgiga.lock"

mkdir -p "$FLAGS_TMP_DIR";
mkdir -p "$FLAGS_FLASH_DIR"
n1=0

#-------------------------------------------------------
# --all function
#-------------------------------------------------------
function all {
    local time=0.01
    local running=true

    while $running; do
    
        if [ ! -f "$GIGAFILE" ]; then running=false; break; fi
        
        echo -ne $LED4321o > "$SPIDEVPATH"
        sleep "$time"
    done

}


#-------------------------------------------------------
# --configfail function
#-------------------------------------------------------
function configfail {
    local time=0.1
    local time2=0.5
    while true; do
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
    done
}

#-------------------------------------------------------
# --final function
#-------------------------------------------------------
function final {

    while true; do
        if [[ -e "$EDGE_DEVICE" ]] || [[ -d "$FOLDER" ]]; 
        then
            echo -ne $LEDedge > "$SPIDEVPATH";
        else
            echo -ne $LEDg > "$SPIDEVPATH";
        fi
        sleep 1
    done
}

#-------------------------------------------------------
# --giga function
#-------------------------------------------------------
function giga {
    local time=0.4

    while true; do
        echo -ne $LEDg > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time"
    done
}

#-------------------------------------------------------
# --testapproved function
#-------------------------------------------------------
function testapproved {
    local time1=2
    local time2=0.18

    while true; do
        echo -ne $LEDg > "$SPIDEVPATH";
        sleep "$time1"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDg > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDg > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
    done
}

#-------------------------------------------------------
# --invalidEDFW function
#-------------------------------------------------------
function invalidEDFW {
    local time1=2
    local time2=0.18

    while true; do
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time1"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
    done
}


#-------------------------------------------------------
# --invalidEDHW function
#-------------------------------------------------------
function invalidEDHW {
    local time1=1.5
    local time2=0.14

    while true; do
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time1"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time2"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time2"
    done
}

#-------------------------------------------------------
# --grav function
#-------------------------------------------------------
function grav {
    local time=1
    local n2=1

    while true; do
        time=$( printf %.2f\\n "$((50 - $n2 ))e-2")			

        if awk "BEGIN {exit !($time <= 0.05)}"; then
            time=0.05
        fi

        echo -ne $LED1g > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED1r > "$SPIDEVPATH";
        sleep "$time"

        echo -ne $LED2g > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED2r > "$SPIDEVPATH";
        sleep "$time"

        echo -ne $LED3g > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED3r > "$SPIDEVPATH";
        sleep "$time"

        echo -ne $LED2g > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED2r > "$SPIDEVPATH";
        sleep "$time"

        n2=$((n2+1))
        if [ "$n2" -ge 49 ]; then n2=49; fi
    done
}

#-------------------------------------------------------
# --init function
#-------------------------------------------------------
function init {
    local time=0.8

    echo -ne $LEDNULL > "$SPIDEVPATH";
    echo -ne $LEDr > "$SPIDEVPATH"
    sleep "$time"

    echo -ne $LEDg > "$SPIDEVPATH"
    sleep "$time"

    echo -ne "\x00" > "$SPIDEVPATH"
}

#-------------------------------------------------------
# --invalid function
#-------------------------------------------------------
function invalid {
    local time=0.4

    while true; do
        echo -ne $LEDNULL > "$SPIDEVPATH";
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LEDNULL > "$SPIDEVPATH";
        sleep "$time"
    done
}

#-------------------------------------------------------
# --loading function
#-------------------------------------------------------
function loading {
    local time=0.3
    local running=true
    touch "$LOCKFILE"

    while $running; do
    
        if [ ! -f "$LOCKFILE" ]; then running=false; break; fi

        echo -ne $LED4o > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED43o > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED432o > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED4321o > "$SPIDEVPATH";
        sleep "$time"
    done

    sleep 1 && init; 

    if [ -f "$LOCKFILE" ]; then rm "$LOCKFILE"; fi
    
}

#-------------------------------------------------------
# --medialeds function
#-------------------------------------------------------
function medialeds {
    local time=0.6
    local running=true

    while $running; do
        #echo -ne $LED4o > "$SPIDEVPATH";
        #sleep "$time"
        #echo -ne $LED41o > "$SPIDEVPATH";
        #sleep "$time"
        #echo -ne $LED42o > "$SPIDEVPATH";
        #sleep "$time"
        #echo -ne $LED43o > "$SPIDEVPATH";
        #sleep "$time"

        echo -ne $LED123o4r > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED124o3r > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED134o2r > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED234o1r > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED134o2r > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED124o3r > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED123o4r > "$SPIDEVPATH";
    done    
}


#-------------------------------------------------------
# --preloading function
#-------------------------------------------------------
function preloading {
    local time=0.3

    echo -ne $LED4o > "$SPIDEVPATH";
    sleep $time
    echo -ne $LED43o > "$SPIDEVPATH";
    sleep $time
    echo -ne $LED432o > "$SPIDEVPATH";
    sleep $time
    echo -ne $LED4321o > "$SPIDEVPATH";

}


#-------------------------------------------------------
# --supportlogs function
#-------------------------------------------------------
function supportlogs {
    local time=0.001
    local count=0
    local max_count=10
    local count_while=0
    local max_count_while=10

    until (( count_while >= max_count_while )) ; do

        count=0
        until (( count >= max_count )); do
            echo -ne $LED4321o > "$SPIDEVPATH"
            sleep $time
            (( count++ ))
        done

        count=0
        until (( count >= max_count )); do
            echo -ne $LEDNULL > "$SPIDEVPATH";
            sleep $time
            (( count++ ))
        done

        (( count_while++ ))
    done
}


#-------------------------------------------------------
# --develop function
#-------------------------------------------------------
function develop {
    local time=0.3

    while true; do
        echo -ne $LED3g > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED23g > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LEDg > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED3o > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED23o > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LEDo > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED3r > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LED23r > "$SPIDEVPATH";
        sleep "$time"
        echo -ne $LEDr > "$SPIDEVPATH";
        sleep "$time"
    done
}


#-------------------------------------------------------------
# STATE 0: TESTANDO CASO NÃO EXISTA O SPIDEV
#-------------------------------------------------------------
# Testa por 60 vezes, 2 vez por segundo, se spidev existe
until [ "$n1" -ge 60 ]
do
	if [[ -e "$SPIDEVPATH" ]]; 
    then 
        n1=100;
        state=1;
    else  
        n1=$((n1+1)) 
        sleep 0.5
	fi
done

#-------------------------------------------------------------
# STATE 1: 
#-------------------------------------------------------------
if [[ "$state" == "1" ]]; 
then
    case "$1" in
            "-h" | "--help" | "")
                echo "" &&
                echo "------------------------------------------------------" &&
                echo "plcleds HELP" &&
                echo "------------------------------------------------------" &&
                echo "--all         : Pisca em alta frequência todos os leds em laranja." &&
                echo "--configfail  : Piscam duas vezes em vermelho e pausam. O ciclo se repete infinitamente. Indica erro no hwconfig." &&
                echo "--init        : Leds estáticos em vermelho e após em verde. Indica a inicialização okay do produto." &&
                echo "--grav        : Leds alternando entre vermelho e verde. Indica que o produto está sendo gravado." &&
                echo "--giga        : Piscam em verde infinitamente. Indica que o produto teve sua primeira gravação okay," &&
                echo "                contudo foi inicializado novamente na giga de gravação." &&
                echo "--final       : Leds estáticos em verde (PLC500/PLC410) ou verde+laranja (PLC500ED). Indica o " &&
                echo "                final do processo de gravação dos produtos." &&
                echo "--invalid     : Piscam em vermelho infinitamente. Indica FW e HW inválidos." &&
                echo "--invalidEDFW : Piscam em vermelho infinitamente. Indica FW do PLC500ED gravado em outro PLC." &&
                echo "--invalidEDHW : Piscam em vermelho infinitamente. Indica HW do PLC500ED gravado com outro FW." &&
                echo "--loading     : Leds carregando em laranja. Indica produto inicializando." &&
                echo "--medialeds   : Leds carregando em um padrão específico. Indica algum processo via media externa." &&
                echo "--preloading  : Leds carregando em laranja até os quatro estarem em laranja." &&
                echo "--supportlogs : Leds piscando em laranja (codesys tb rodando)." &&
                echo "" &&
                echo "--develop     : Leds alternam verde, laranja e vermelho" &&
                echo "--testapproved: Leds piscando em verde (plc aprovado no teste)." &&
                echo "------------------------------------------------------" 
                ;;

                
            "--all")
                $(all)
                ;;

            "--configfail")
                $(configfail)
                ;;

            "--final")
                $(final)
                ;;

            "--giga")
                $(giga)
                ;;

            "--grav")
                $(grav)
                ;;

            "--init")
                $(init)
                ;;

            "--invalid")
                $(invalid)
                ;;

            "--invalidEDFW")
                $(invalidEDFW)
                ;;

            "--invalidEDHW")
                $(invalidEDHW)
                ;;

            "--loading")
                $(loading)
                ;;

            "--medialeds")
                $(medialeds)
                ;;

            "--preloading")
                $(preloading)
                ;;

            "--supportlogs")
                $(supportlogs)
                ;;

            "--testapproved")
                $(testapproved)
                ;;

            "--develop")
                $(develop)
                ;;

            *)
                echo "No option provided. Try -h to see the help."
    esac
fi

