#!/bin/bash

#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh

#-------------------------------------------------------------
# Help
#-------------------------------------------------------------
if [[ "$1" == "-h" || "$1" == "--help"   ]]; then 

    echo ""
    echo "----------------------------------------------"
    echo "PLCSNIFFER HELP"
    echo "----------------------------------------------"
    echo ""
    echo "Configure sniffer"
    echo "./plcsniffer.sh <interface> <dest_IP>"
    echo "   interface: 0 = eth1 and eth2, 1 = eth1, 2 = eth2, 3 = br1"
    echo "   dest_IP: must be the destination host address . Ex: 192.168.2.100 (from the PC)."
    echo ""
    echo "Removing rules" 
    echo "./plcsniffer.sh -r"
    echo ""
    echo "Check rules applied" 
    echo "./plcsniffer.sh -i"
    echo ""
    echo "----------------------------------------------"
    echo ""

    exit 0
fi

#-------------------------------------------------------------
# Check rules applied
#-------------------------------------------------------------
if [[ "$1" == "-i" ]]; then 

    HOSTNAME=$(hostname)

    echo ""
    echo "----------------------------------------------"
    echo "PLCSNIFFER RULES APPLIED"

    if [[ "$HOSTNAME" == "PLC500ED" ]]; then
        echo "iptables -t mangle -L -v"
        echo "----------------------------------------------"
        echo ""
        iptables -t mangle -L -v
        echo ""
    else
        echo "iptables-legacy -t mangle -L -v"
        echo "----------------------------------------------"
        echo ""
        iptables-legacy -t mangle -L -v
    fi

    echo ""
    echo "----------------------------------------------"
    echo ""

    exit 0
fi

#-------------------------------------------------------------
# Deconfiguring Sniffer
#-------------------------------------------------------------
if [[ "$1" == "-r" ]]; then 

    HOSTNAME=$(hostname)

    if [[ "$HOSTNAME" == "PLC500ED" ]]; then
        iptables -t mangle -F PREROUTING
        iptables -t mangle -F POSTROUTING
        iptables -t mangle -F FORWARD
        iptables -t mangle -F INPUT
        iptables -t mangle -F OUTPUT
    else
        iptables-legacy -t mangle -F PREROUTING
        iptables-legacy -t mangle -F POSTROUTING
        iptables-legacy -t mangle -F FORWARD
        iptables-legacy  -t mangle -F INPUT
        iptables-legacy  -t mangle -F OUTPUT
    fi

    exit 0
fi

#-------------------------------------------------------------
# Configuring Sniffer
#-------------------------------------------------------------
HOSTNAME=$(hostname)
INTERFACE="$1"
DEST_IP="$2"

# INTERFACE: 0 = ETH1 e ETH2, 1 = ETH1, 2 = ETH2

if [[ -z "$INTERFACE" || ("$INTERFACE" != "0" && "$INTERFACE" != "1" && "$INTERFACE" != "2" && "$INTERFACE" != "3" ) ]]; then
    echo "----------------------------------------------"
    echo "ERROR!"
    echo "Please try again and enter the correct argument or interface to sniffer"
    echo "----------------------------------------------"
    exit 1
fi

# Habilitando o roteamento entre interfaces
echo 1 > /proc/sys/net/ipv4/ip_forward

# Code according to the device
if [[ "$HOSTNAME" == "PLC500ED" ]]; then

    if [[ "$INTERFACE" == "0" ]]; then
        iptables -t mangle -A PREROUTING -i "eth1" -j TEE --gateway "$DEST_IP"
        iptables -t mangle -A PREROUTING -i "eth2" -j TEE --gateway "$DEST_IP"

        iptables -t mangle -A POSTROUTING -o "eth1" -j TEE --gateway "$DEST_IP"
        iptables -t mangle -A POSTROUTING -o "eth2" -j TEE --gateway "$DEST_IP"
    fi

    if [[ "$INTERFACE" == "1" ]]; then
        iptables -t mangle -A PREROUTING -i "eth1" -j TEE --gateway "$DEST_IP"
        iptables -t mangle -A POSTROUTING -o "eth1" -j TEE --gateway "$DEST_IP"
    fi

    if [[ "$INTERFACE" == "2" ]]; then
        iptables -t mangle -A PREROUTING -i "eth2" -j TEE --gateway "$DEST_IP"
        iptables -t mangle -A POSTROUTING -o "eth2" -j TEE --gateway "$DEST_IP"
    fi

    if [[ "$INTERFACE" == "3" ]]; then
        modprobe br_netfilter
        iptables -t mangle -A PREROUTING -i "br1" -j TEE --gateway "$DEST_IP"
        iptables -t mangle -A POSTROUTING -o "br1" -j TEE --gateway "$DEST_IP"
    fi

else
    if [[ "$INTERFACE" == "0" ]]; then
        iptables-legacy -t mangle -A PREROUTING -i "eth1" -j TEE --gateway "$DEST_IP"
        iptables-legacy -t mangle -A PREROUTING -i "eth2" -j TEE --gateway "$DEST_IP"

        iptables-legacy -t mangle -A POSTROUTING -o "eth1" -j TEE --gateway "$DEST_IP"
        iptables-legacy -t mangle -A POSTROUTING -o "eth2" -j TEE --gateway "$DEST_IP"
    fi

    if [[ "$INTERFACE" == "1" ]]; then
        iptables-legacy -t mangle -A PREROUTING -i "eth1" -j TEE --gateway "$DEST_IP"
        iptables-legacy -t mangle -A POSTROUTING -o "eth1" -j TEE --gateway "$DEST_IP"
    fi

    if [[ "$INTERFACE" == "2" ]]; then
        iptables-legacy -t mangle -A PREROUTING -i "eth2" -j TEE --gateway "$DEST_IP"
        iptables-legacy -t mangle -A POSTROUTING -o "eth2" -j TEE --gateway "$DEST_IP"
    fi

    if [[ "$INTERFACE" == "3" ]]; then
        modprobe br_netfilter
        iptables-legacy -t mangle -A PREROUTING -i "br1" -j TEE --gateway "$DEST_IP"
        iptables-legacy -t mangle -A POSTROUTING -o "br1" -j TEE --gateway "$DEST_IP"
    fi

fi
