#!/bin/bash


#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh


#-------------------------------------------------------------
# Debug echo Function
#-------------------------------------------------------------
function debug_echo {
    if [[ -n "$DEBUG_FLAG" ]]; then           # Exibe a mensagem apenas se DEBUG_FLAG estiver definida
        echo "DEBUG_WIFI: $1" | tee /dev/kmsg;   
    fi
}

#-------------------------------------------------------------
# Try then test Function
#-------------------------------------------------------------
function try_then_execute {
    local test="$1"
    local command="$2"
    local tries="$3"
    local count=0

    debug_echo "try_then_execute: $test"

    if eval "$test"; then

        if [[ -z "$tries" ]]; then tries=10; fi

        while [[ "$count" -lt "$tries" ]]; do
            eval "$command" || true; 
            if ! eval "$test"; then count=100; debug_echo "rodou de primeira"; return 0; fi
            ((count++))
            sleep 1
            debug_echo "trying to execute the command again..."
            if [[ "$count" -ge "$tries" ]]; then
                return 1
            fi
        done
        return 0
    else
        debug_echo "false condition"
        return 0
    fi 
}

#-------------------------------------------------------------
# Configuração inicial
#-------------------------------------------------------------
function init_config {

    debug_echo "entrou init_config"

    if [[ "$HW_VERSION" != "3" ]] && [[ "$HW_VERSION" != "0" ]]; then

        if ! [[ -f "$plcconfig" ]];        then plcLog ERROR não existe o arquivo "$plcconfig"; fi
        if ! [[ -f "$plcconfig_backup" ]]; then plcLog ERROR não existe o arquivo "$plcconfig_backup"; fi

        if ! [[ -f "$plcconfig" ]] && [[ -f "$plcconfig_backup" ]]; 
        then 
            cp "$plcconfig_backup" "$plcconfig"
        fi

        # plconfig: SSID e PASSWD (primeira escrita no arquivo)
        if grep -q 'AP_SSID = ""' "$plcconfig" || grep -q 'AP_PASSWD = ""' "$plcconfig"; then 
            mac1=$(cat /run/weg/json/networks.json | grep -o '"MAC1": "[^"]*' | cut -d '"' -f 4)
            plcmac=$(echo $mac1 | sed -E 's/.*(..:..:..).*/\1/')
            plcpasswd=$(echo "$plcmac" | tr -d ':')

            sed -i "s/AP_SSID.*/AP_SSID = \"$(hostname):$plcmac\"/" "$plcconfig"
            sed -i "s/AP_PASSWD.*/AP_PASSWD = \"weg$plcpasswd\"/" "$plcconfig"
        fi
    fi
}

#-------------------------------------------------------------
# Load 88x2bu driver
#-------------------------------------------------------------
function load_driver {

    debug_echo "entrou load_driver"

    if [[ "$HW_VERSION" == "0" ]]; then exit 0; fi

    try_then_execute "! lsmod | grep -wq 88x2bu" "modprobe 88x2bu" || plcLog ERROR wifi driver could not be installed

}

#-------------------------------------------------------------
# Read plcconfig
#-------------------------------------------------------------
function read_plcconfig {

    debug_echo "entrou read_plcconfig"

    wifi_type=$(awk -F '= ' '/WIFI_TYPE =/{print $2}' "$plcconfig")
    web_type=$(awk -F '= ' '/WEB_TYPE =/{print $2}' "$plcconfig")
    web_ssid=$(grep 'WEB_SSID =' "$plcconfig" | awk -F '"' '{print $2}')
    web_passwd=$(grep 'WEB_PASSWD =' "$plcconfig" | awk -F '"' '{print $2}')
    web_login=$(grep 'WEB_LOGIN =' "$plcconfig" | awk -F '"' '{print $2}')

    ap_ssid=$(grep 'AP_SSID =' "$plcconfig" | awk -F '"' '{print $2}')
    ap_passwd=$(grep 'AP_PASSWD =' "$plcconfig" | awk -F '"' '{print $2}')
}

#-------------------------------------------------------------
# Set PLC500 Wifi
#-------------------------------------------------------------
function set_plc500 {

    debug_echo "entrou set_plc500"


    # Se configurado como internet mode
    if [[ "$wifi_type" == "1" ]]; then 

        debug_echo "init wifi_type = 1"

        # Open
        if [[ "$web_type" == "0" ]]; then 
            debug_echo "init open"
            web_open "$web_ssid"

        # PSK
        elif [[ "$web_type" == "1" ]]; then 
            debug_echo "init psk"
            web_psk "$web_ssid" "$web_passwd"

        # Corporate
        elif [[ "$web_type" == "2" ]]; then 
            debug_echo "init corp"
            web_corp "$web_login" "$web_ssid" "$web_passwd" "add"

        fi

    else 
        debug_echo "init wifi_type = 0"
    fi
}

#-------------------------------------------------------------
# Set PLC500ED
#-------------------------------------------------------------
function set_plc500ed {

    debug_echo "entrou set_plc500ed"

    if [[ "$HW_VERSION" == "3" ]];
    then

        echo "5" > "$edge_device"; 

        #if [[ -f "${config_path}" ]] && grep -q "\[interfaces.wlan0\]" "${config_path}"; then

            # Calcula o numero da linha da interface wlan0
            #linha=$(grep -n -m 1 "$wlan_toml" "$config_path" | cut -d: -f1)

            # Seta wlan0 como ativo no config.toml 
            # sed -i "${linha},\$ s/active = true/active = false/g" "${config_path}"

        #fi
    fi

}

#-------------------------------------------------------------
# Set PLC500ED-WiFi
#-------------------------------------------------------------
function set_plc500edw {

    debug_echo "entrou set_plc500edw"

    if [[ "$HW_VERSION" == "3" ]];
    then

        echo "6" > "$edge_device"; 

        #if [[ -f "$config_path" ]]; then

            # Calcula o numero da linha da interface wlan0
            #linha=$(grep -n -m 1 "$wlan_toml" "$config_path" | cut -d: -f1)

            # Captura o conteúdo de "ssid = " após [interfaces\.wlan0\]
            #ssid=$(awk '/\[interfaces\.wlan0\]/ {flag=1; next} flag && /ssid = / {print $3; exit}' "${config_path}")

            #if [[ "$ssid" == "" || -z "$ssid" ]]; then 
            #    sed -i "${linha},\$ s/active = false/active = true/g" "${config_path}"
            #fi

        #fi

        plcLog INFO dongle Wi-Fi "$ID_USB_VENDOR:$ID_USB_PRODUCT" connected; 

    fi

}

#-------------------------------------------------------
# Verifica wlan0 e executa weg-coreapp para PLC500ED
#-------------------------------------------------------
function init_coreapp {

    if [[ "$HW_VERSION" == "3" ]];
    then
        debug_echo "entered init_coreapp"

        nohup systemctl restart weg-coreapp > /dev/null 2>&1 & 
    fi
}

#-------------------------------------------------------
# Verify wifi interface
#-------------------------------------------------------
function verify_wifi {

    debug_echo "entered verify_wifi"

    local count=0
    local tries=15

    if [[ "$HW_VERSION" == "0" ]]; then exit 0; fi

    if [[ "$HW_VERSION" == "3" ]];
    then

        while (( count <= tries )); do
            if ip link | grep -wq "wlan0"; then
                ip link set wlan0 up

                DEVICE=$(cat "$edge_device")

                if [[ "$DEVICE" == "6" ]]; then
                    return 1
                else
                    echo "6" > "$edge_device";  
                    debug_echo "iniciando PLC500EDW"
                    return 0; 
                fi

            else
                ((count++))
                sleep 0.5
                debug_echo "count = $count"
                if [[ "$count" -ge "$tries" ]]; 
                then 
                    debug_echo "counter >= $tries.. iniciando PLC500ED"; 
                    echo "5" > "$edge_device"; 
                    return 0; 
                fi
            fi
        done
    else
        if ip link | grep -q "wlan0"; then
            ip link set wlan0 up; ret=$?

            if [[ "$ret" == "0" ]]; then 
                
                plcLog INFO dongle Wi-Fi "$ID_USB_VENDOR:$ID_USB_PRODUCT" connected; 
                
                sed -i "s/DONGLE_CONN.*/DONGLE_CONN = 1/" "$plcconfig"
                
            fi
        fi
    fi
}


#-------------------------------------------------------------
# Verifica se o dongle Wi-Fi foi conectado
#-------------------------------------------------------------
function verify_wlan {

    debug_echo "entrou verify_wlan"

    if ip link | grep -q "wlan0"; then

        # Sobe a interface wlan0
        ip link set wlan0 up; ret=$?

        if [[ "$ret" == "0" ]]; then 
            
            plcLog INFO dongle Wi-Fi "$ID_USB_VENDOR:$ID_USB_PRODUCT" connected; 

            if [[ "$HW_VERSION" != "3" ]]; then
                sed -i "s/DONGLE_CONN.*/DONGLE_CONN = 1/" "$plcconfig"
            fi

        else 
            plcLog ERROR error trying to set wlan0 up "$ID_USB_VENDOR:$ID_USB_PRODUCT"; exit 0;
        fi

    else
        set_plc500ed; exit 0;

    fi
}

#-------------------------------------------------------------
# Internet Connection - (PLC500 e PLC500MC) - Rede Aberta - ssid
#-------------------------------------------------------------
function web_open {
    debug_echo "entrou no web_open"

    # kill wpa_supplicant and dhclient
    if pgrep -x "wpa_supplicant" > /dev/null; then kill $(pgrep wpa_supplicant); fi
    dhclient -r wlan0;

    # atualiza wpa_supplicant
    open_config="network={
        ssid=\"$1\"
        key_mgmt=NONE
    }"

    echo "$open_config" > "$wpa_conf"

    # atualiza plcconfig
    sed -i "s/WIFI_TYPE.*/WIFI_TYPE = 1/" "$plcconfig"
    sed -i "s/WEB_TYPE.*/WEB_TYPE = 0/" "$plcconfig"
    sed -i "s/WEB_SSID.*/WEB_SSID = \"$1\"/" "$plcconfig"

    # Inicia o wpa_supplicant e tenta adquirir o IP (para todos os modos)
    dongle_connected=$(awk -F '= ' '/DONGLE_CONN =/{print $2}' "$plcconfig")

    if [[ "$dongle_connected" -eq "1" ]]; then 
        nohup wpa_supplicant -B -i wlan0 -c "$wpa_conf" >/dev/null 2>&1 &
        nohup dhclient wlan0 >/dev/null 2>&1 &
        debug_echo "entrou wpa e dhclient" 
    fi



}

#-------------------------------------------------------------
# Internet Connection - (PLC500 e PLC500MC) - Rede PSK - ssid, password
#-------------------------------------------------------------
function web_psk {
    debug_echo "entrou no web_psk"

    # kill wpa_supplicant and dhclient
    if pgrep -x "wpa_supplicant" > /dev/null; then kill $(pgrep wpa_supplicant); fi
    dhclient -r wlan0;

    # atualiza wpa_supplicant
    wpa_passphrase "$1" "$2" > "$wpa_conf"

    # deleta a linha comentada com a senha
    sed -i '/^\s*#psk/d' $wpa_conf

    # psk password
    psk_passwd=$(grep 'psk=' "$wpa_conf" | sed 's/.*psk=//')

    # atualiza plcconfig
    sed -i "s/WEB_TYPE.*/WEB_TYPE = 1/" "$plcconfig"
    sed -i "s/WEB_SSID.*/WEB_SSID = \"$1\"/" "$plcconfig"
    sed -i "s/WEB_PASSWD.*/WEB_PASSWD = \"$psk_passwd\"/" "$plcconfig"

    # Inicia o wpa_supplicant e tenta adquirir o IP (para todos os modos)
    dongle_connected=$(awk -F '= ' '/DONGLE_CONN =/{print $2}' "$plcconfig")

    if [[ "$dongle_connected" -eq "1" ]]; then 
        nohup wpa_supplicant -B -i wlan0 -c "$wpa_conf" >/dev/null 2>&1 &
        nohup dhclient wlan0 >/dev/null 2>&1 &
        debug_echo "entrou wpa e dhclient"
    fi
}

#-------------------------------------------------------------
# Internet Connection - (PLC500 e PLC500MC) - Rede corporativa - identity, ssid, password
#-------------------------------------------------------------
function web_corp {
    debug_echo "entrou no web_corp"

    # kill wpa_supplicant and dhclient
    if pgrep -x "wpa_supplicant" > /dev/null; then kill $(pgrep wpa_supplicant); fi
    dhclient -r wlan0;

    # hash diretamente do plcconfig ou calcula?
    if [[ "$4" == "add" ]]; then
        hash_md4=$3

    else 
        # calcula a hash da senha
        hash_md4=$(echo -n "$3" | iconv -t utf16le | openssl dgst -md4 | awk '{print $2}')

    fi

    # atualiza wpa_supplicant
    corp_config="p2p_disabled=1
                network={
                    ssid=\"$2\"
                    key_mgmt=WPA-EAP
                    eap=TTLS
                    identity=\"$1\"
                    password=hash:$hash_md4
                    proto=RSN
                    auth_alg=OPEN
                    pairwise=CCMP
                    phase1=\"peaplabel=0\"
                    phase2=\"auth=MSCHAPV2\"
                }"


    echo "$corp_config" > "$wpa_conf"

    # atualiza plcconfig
    sed -i "s/WEB_TYPE.*/WEB_TYPE = 2/" "$plcconfig"
    sed -i "s/WEB_LOGIN.*/WEB_LOGIN = \"$1\"/" "$plcconfig"
    sed -i "s/WEB_SSID.*/WEB_SSID = \"$2\"/" "$plcconfig"
    sed -i "s/WEB_PASSWD.*/WEB_PASSWD = \"$hash_md4\"/" "$plcconfig"

    # Inicia o wpa_supplicant e tenta adquirir o IP (para todos os modos)
    dongle_connected=$(awk -F '= ' '/DONGLE_CONN =/{print $2}' "$plcconfig")

    if [[ "$dongle_connected" -eq "1" ]]; then 
        nohup wpa_supplicant -B -i wlan0 -c "$wpa_conf" >/dev/null 2>&1 &
        nohup dhclient wlan0 >/dev/null 2>&1 &
        debug_echo "entrou wpa e dhclient"
    fi

}


#-------------------------------------------------------------
# Paths
#-------------------------------------------------------------
config_path="/etc/core-app/config.toml"
edge_device="/var/lib/edge_device"
hostapd_conf="/etc/hostapd/hostapd.conf"
new_active="active = false"
new_default="default = false"
plcconfig="/etc/weg/plcconfig.toml"
plcconfig_backup="/opt/weg/backups/plcconfig.toml"
wlan_toml="interfaces.wlan0"
wlan_interf="/etc/network/interfaces.d/wlan0"
wpa_conf="/etc/wpa_supplicant/wpa_supplicant-wlan0.conf"


#-------------------------------------------------------------
# Main program
#-------------------------------------------------------------
#DEBUG_FLAG=1
HW_VERSION=$(grep -oP '(?<="HW_VERSION": )[0-9]+' /etc/weg/firstBoot.json)

if [[ "$HW_VERSION" == "0" ]]; then exit 0; fi
if [[ "$ID_USB_VENDOR" == "1d6b" ]]; then exit 0; fi
if [[ "$ID_USB_VENDOR" == "0424" ]]; then exit 0; fi


#-------------------------------------------------------------
# Add the dongle
#-------------------------------------------------------------
if [[ "$1" == "--add" ]] && ! ls /dev/sd* &> /dev/null; then

    debug_echo "entrou --add"

    load_driver 
    verify_wlan

    if [[ "$HW_VERSION" == "3" ]];
    then
        set_plc500edw
        init_coreapp
    else
        #read_plcconfig
        plcLog INFO dongle Wi-Fi "$ID_USB_VENDOR:$ID_USB_PRODUCT" connected; 
    fi


#-------------------------------------------------------------
# Init (plcinit)
#-------------------------------------------------------------
elif [[ "$1" == "--init" ]] && ! ls /dev/sd* &> /dev/null; then

    debug_echo "entrou --init"

    load_driver 
    init_config
    verify_wlan

    if [[ "$HW_VERSION" == "3" ]];
    then
        set_plc500edw
    else
        read_plcconfig

        sed -i "s/DONGLE_CONN.*/DONGLE_CONN = 0/" "$plcconfig"
    fi


#-------------------------------------------------------------
# Internet Connection - (PLC500 e PLC500MC) - Rede Aberta - ssid
#-------------------------------------------------------------
elif [[ "$1" == "--open" && "$HW_VERSION" != "3" && -n "$2" ]]; 
then
    debug_echo "entrou no --open"
    web_open "$2"


#-------------------------------------------------------------
# Internet Connection - (PLC500 e PLC500MC) - Rede PSK - ssid, password
#-------------------------------------------------------------
elif [[ "$1" == "--psk" && "$HW_VERSION" != "3" && -n "$2" && -n "$3" ]]; 
then
    debug_echo "entrou no --psk"
    web_psk "$2" "$3"


#-------------------------------------------------------------
# Internet Connection - (PLC500 e PLC500MC) - Rede corporativa - identity, ssid, password
#-------------------------------------------------------------
elif [[ "$1" == "--corp" && "$HW_VERSION" != "3" && -n "$2" && -n "$3" && -n "$4" ]]; 
then
    debug_echo "entrou no --corp"
    web_corp "$2" "$3" "$4" 


#-------------------------------------------------------------
# Access Point - (PLC500 e PLC500MC)
#-------------------------------------------------------------
elif [[ "$1" == "--ap" && "$HW_VERSION" != "3" ]]; 
then

    # Perde o IP de wlan0
    dhclient -r wlan0

    if [[ "$2" == "--save" && "$3" != "" ]]; then
        sed -i "s/AP_PASSWD.*/AP_PASSWD = \"$3\"/" "$plcconfig"
    fi

    # Inicializa hostapd e configura IP
    systemctl start hostapd
    ifconfig wlan0 10.10.10.10 netmask 255.255.255.0


#-------------------------------------------------------------
# Wifi Disabled - (PLC500 e PLC500MC)
#-------------------------------------------------------------
elif [[ "$1" == "--disabled" && "$HW_VERSION" != "3" ]]; 
then

    # Se hostapd está sendo executado, fechar
    # systemctl stop hostapd; pgrep -x "hostapd" && killall hostapd

    # Perde o IP de wlan0 e mata o dhclient
    dhclient -r wlan0


#-------------------------------------------------------------
# Remove the dongle
#-------------------------------------------------------------
elif [[ "$1" == "--rm" ]]; 
then
    debug_echo "entrou no --rm"

    if [[ "$HW_VERSION" == "3" ]]; then 

        DEVICE=$(cat "$edge_device")

        if [[ "$DEVICE" == "6" ]]; then
            set_plc500ed
            init_coreapp
        fi
    fi

    # Para executar
    #contador=0
    #until [[ $contador -gt 10 ]] || [[ $contador -gt 2 && $wlan -eq 0 ]]; do

    #    if [[ -d "/sys/bus/usb/devices/2-1" ]]; then dongle=1; else dongle=0; fi
    #    if ip link | grep -q "wlan0"; then wlan=1; else wlan=0; fi
    #    sleep 1

    #    ((contador++))
    #done

    # Se hostapd está sendo executado, fechar
    # systemctl stop hostapd; pgrep -x "hostapd" && killall hostapd

    # kill wpa_supplicant and dhclient
    #if pgrep -x "wpa_supplicant" > /dev/null; then kill $(pgrep wpa_supplicant); fi
    #dhclient -r wlan0;

#-------------------------------------------------------------
# Verify wifi (sobe módulo) and init coreapp
#-------------------------------------------------------------
elif [[ "$1" == "--wifi_coreapp" ]]; 
then
    load_driver
    verify_wifi && init_coreapp

    

fi