from webserver.login import Session
from webserver.util import Utilities
from webserver.variables import urlsHandles, htmlText, additionalPages, globalVariables, filesPaths
from http.server import SimpleHTTPRequestHandler
from http.cookies import SimpleCookie
from cgi import FieldStorage
import datetime
from functools import wraps


class PLCServer(SimpleHTTPRequestHandler, Session, Utilities):
    def __init__(self, *args, **kwargs):
        self.cookie = SimpleCookie()
        self.login_fields = htmlText.login_fields
        super().__init__(*args, **kwargs)

    def parse_cookies(self, cookie_list):
        """
        return a dict from cookies in the form 'SimpleCookie: key=data'
        """
        return dict(((c.strip().split("=")) for c in cookie_list.split(";"))) \
        if cookie_list else {}
    
    # funcao para remover cookies auxiliares do navegador
    def remove_cookies(self, tipo):
        if tipo == 'text/html':
            if self.path != '/firmwareupdate.html':
                self.cookie.load({'update': 'None', 'boot':'None'})
            if self.path != '/configuration.html':
                self.cookie.load({'ftpid': 'None'})
                self.cookie.load({'TemporaryIP': 'None'})

    # checando os cookies fornecidos pelo cliente
    def check_session_auth(self, cookie_check) -> bool:
        """
        check session and update html format parameters
        """
        # TODO possivel falha de segurança e deve ser refeito
        # self.login_fields.update(htmlText.login_dict[cookie_check.get('error', 'Login')])
        if 'error' in cookie_check:
            try:
                self.login_fields.update(htmlText.login_dict[cookie_check['error']])

            except KeyError:    # falha de segurança
                self.login_fields.update(htmlText.login_dict['Login'])

        if "sid" in cookie_check:
            if cookie_check["sid"] != 'None':
                self.cookie_stored = self.retrive_id_cookie(globalVariables.path_session, cookie_check["sid"])

                if 'login_step' in self.cookie_stored: # check if exists
                    self.login_fields.update(self.cookie_stored['login_step'])

                if cookie_check['sid'] in self.cookie_stored:
                    self.current_cookie = cookie_check
                    return True
            return False
        else:
            return False

    def session_check(func):
        """
        function decorator to verify credentials
        """
        @wraps(func)
        def requests_credential(self):
            cookie_check = self.parse_cookies(self.headers["Cookie"])

            if not self.check_session_auth(cookie_check):
                if self.guess_type(self.path) == "text/html" or self.path == "/":
                    self.path = "/login.html"
                if self.path not in urlsHandles.URLS_ALLOWED_ALWAYS:
                    self.path = "/login.html"

            func(self)
        return requests_credential

    @session_check
    def do_HEAD(self):
        """
        http.server builtin function to deal with HEAD requests
        """
        # if self.command == 'HEAD':
        self.log_request(200)
        self.send_response_only(200)
        self.send_header("Content-Length", str(256))
        output_date = datetime.datetime.now().strftime("%m/%d/%Y %H:%M:%S")
        self.send_header('Date', output_date)
        self.end_headers()

    @session_check
    def do_GET(self):
        """
        http.server builtin function to deal with GET requests
        """
        # caso for para arquivos para o navegador 
        # e nao para baixar os logs
        # TODO encontrar variaveis estaticas e fazer
        # TODO certeza de que nao criar dinamicamente
        if self.path not in filesPaths.files_zip.keys():
            # quando digitar o IP deve acessar status.html
            if self.path == ('/'):
                self.path = '/status.html'

            self.tipo = self.guess_type(self.path)

            try:
                self.full_path = self.find_path(self.path, self.tipo)
            except KeyError:
                self.log_error(f'File format not available {self.path, self.tipo}')

            # print(self.tipo, self.full_path, self.verify_exists(self.full_path))

            # remoção de cookies auxiliares entre as paginas
            self.remove_cookies(self.tipo)

            globalVariables.firm_update, globalVariables.boot_update = self.commands_after_post(globalVariables.firm_update, globalVariables.boot_update, self.tipo)

            # verificar se eh arquivo valido
            try:
                self.verify_exists(self.full_path)
                self.handle_file(self.full_path)
            # Handle errors, funciona para arquivos e diretorios
            except FileNotFoundError as msg:
                self.handle_error(msg)

        else:
            if self.path == '/getdevlog':
                self.exec_bash(f"{filesPaths.path_scripts['plcsecrets']} ", '')
            path_zip = self.zip_logs(filesPaths.files_zip[self.path], filesPaths.log_files_extension[self.path])
            path = path_zip[1]
            if self.path == '/getdevlog':
                path = self.encrypt_zip_file_openssl(path)
            self.handle_file(path)
            
            self.remove_file('/tmp/tmp*')
            self.remove_file('/tmp/sec*')


    @session_check
    def do_POST(self):
        """
        http.server builtin function to deal with POST requests
        """
        form = FieldStorage(fp=self.rfile,
                        headers=self.headers,
                        environ={'REQUEST_METHOD': 'POST'}, 
                        encoding='utf-8', 
                        max_num_fields=10
                        )
        # print(form.keys())

        # teste se campos sao validos e menor do que dois
        fields = form.keys()
        if all([i in htmlText.form_fields for i in fields]):

            if self.path == '/login.html':

                if any(key in ['Password', 'Cancel'] for key in fields):
                    # self.empty_sessions()
                    res = self.login_steps(form)
                    self.cookie.load(res)

            # se nao eh um deve ser o outro
            elif self.path != '/login.html':
                retorno = self.logout_change(fields)
                if retorno is not None:
                    self.path = "/login.html"
                    self.cookie.load(retorno)

                if self.path == '/firmwareupdate.html':
                    self.file_download(form)
                    globalVariables.firm_update, globalVariables.boot_update = self.exec_update(fields)
                    self.cookie.load({'update': globalVariables.firm_update, 'boot':globalVariables.boot_update})
                else:
                    self.cookie.load({'update': 'None', 'boot':'None'})

                self.cookie.load({'ftpid':'None'})

                if self.path == '/configuration.html':
                    self.cookie.load({'ftpid':self.handle_config(form)})
                    self.cookie.load({'TemporaryIP': globalVariables.temporary_ip})
                    globalVariables.temporary_ip = None
                else:
                    self.cookie.load({'TemporaryIP': 'None'})
            else:
                self.path = '/login.html'
                self.empty_sessions()

        else:   # caso ultrapassar numero permitido de dados dos formularios
            self.log_error(f'Submitted form keys that are not supported by this server: \r\n{form.keys(), form.length}')
            # self.empty_sessions()        # remover qualquer login anterior
            self.path = "/login.html"

        # rediretion for PRG pattern
        self.tipo = self.guess_type(self.path)
        self.full_path = self.find_path(self.path, self.tipo)
        self.do_REDIRECT(self.full_path)

    def do_REDIRECT(self, full_path):
        """
        return the same file but with code for redirection
        """
        # 301 will redirect with a GET request even if you sent a POST request
        # https://stackoverflow.com/questions/2506932/how-do-i-redirect-a-request-to-a-different-url-in-python
        self.handle_file(full_path, 301)

    def handle_file(self, full_path, status_code=200):
        try:
            content = self.format_content(full_path)
            self.send_content(content, status_code)
        except IOError as msg:
            msg = f"'{self.path}' cannot be read: {msg}"
            self.handle_error(msg)

    def format_content(self, full_path):
        """
        funcao para manipular os dados dentro dos arquivos enviados
        """
        with open(full_path, 'rb') as reader:
            content = reader.read()
        # modificar apenas templates html
        if self.guess_type(self.path) == 'text/html':
            # format ja pega cada "kwarg" do texto
            return ((content.decode()).format(**self.login_fields)).encode()
        else:
            return content

    def handle_error(self, msg):
        """
        geracao e envio da pagina de error do servidor web
        """
        # self.empty_sessions()
        self.cookie.load({'sid': 'None', 'error': 'None'})
        # content = additionalPages.ERRORPAGE.format(path=self.path, msg=msg)
        self.log_request(404)
        self.send_response_only(404)
        self.send_header('Date', self.date_time_string())
        # self.wfile.write(content.encode())

    def send_content(self, content, status=200):
        self.log_request(status)
        self.send_response_only(status)
        self.set_cookies()
        self.content_length = len(content)
        self.set_headers()
        self.wfile.write(content)

    # removido de do_HEAD() para facilitar controle pelo LOGIN e COOKIES
    def set_headers(self):
        """
        configura headers de acordo com o arquivo e requisicao
        """
        self.send_header("Content-Length", str(self.content_length))
        self.send_header('Location', self.path)
        self.send_header("Content-type", self.guess_type(self.path))
        output_date = datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S")
        self.send_header('Date', output_date)
        self.end_headers()

    def set_cookies(self):
        if self.cookie.keys() is None:
            self.cookie = SimpleCookie()
        for item in self.cookie.keys():
            self.send_header("Set-Cookie", f"{item}={self.cookie[item].value}")

    def logout_change(self, keys:list) -> None:
        """
        deal with requests form inside of webpage to login page
        """
        if "Logout" in keys:
            self.html_login_form('Login', 'button_forgot')
            try: # apenas tendo certeza absoluta de remover msm sem SID
                self.remove_id_session(globalVariables.path_session, self.current_cookie["sid"])
            except:
                self.log_message(f'INFO plcwebpage: Foi tentando Logout sem cookie configurado: \r\n{self.headers}')
            return {'sid': 'None', 'error': 'None'}

        if "Change" in keys:
            self.html_login_form('login_verify', 'button_cancel_verify')
            return {'error': 'None'} # 'hsid': hashlib.sha256(f'{hash("Change")}'.encode()).hexdigest(), 

    def login_steps(self, form: FieldStorage):
        """
        passos de verificacao e troca de senhas
        -> para troca de senha é necessário credentiais validas
        """
        if form.getvalue("Login"):
            if self.test_input(form, 'Login') != None:
                user_login, pass_login = self.test_input(form, "Login")

                if self.hash_compare_saved([user_login, pass_login]) or self.forgot_pass(user_login, pass_login):
                    handler = {'error': 'None'} # 'hsid': hashlib.sha256(f'{"Login"}'.encode()).hexdigest(), 
                    self.set_id_and_store(globalVariables.path_session)
                    self.path = "/status.html"
                    self.log_message(f'INFO plcwebpage: Novo acesso ao webserver concedido: \r\n{self.headers}')

                else:
                    self.log_message(f'ERROR plcwebpage: Acesso ao webserver negado: {user_login, self.client_address} \r\n{self.headers}')
                    handler = {'error': 'login_error'}

                return handler
            else:
                return {'error': 'login_error'}
        
        if 'Cancel' in form:
            self.html_login_form('Login', 'button_forgot')
            self.login_fields.update(htmlText.login_dict['Login'])
            self.empty_sessions()
            
            return {'error': 'None'}

        elif form.getvalue("login_verify"):
            if self.test_input(form, 'login_verify') != None:
                user_login, pass_login = self.test_input(form, "login_verify")

                if self.hash_compare_saved([user_login, pass_login]) or self.forgot_pass(user_login, pass_login):
                    handler = {'error': 'None'} # 'hsid': hashlib.sha256(f'{self.current_cookie["hsid"]}'.encode()).hexdigest(), 
                    step = 'new_login'

                else:
                    handler = {'error': 'login_error'}
                    step = 'login_verify_error'

                self.html_login_form(step, 'button_cancel_new')
                self.login_fields.update({'msgerror': 'Invalid credentials'})

                return handler

            else:
                return {'error': 'login_error'}

        elif form.getvalue("new_login"):
            if self.test_input(form, 'new_login') != None:
                user_login, pass_login = self.test_input(form, "new_login")
                self.html_login_form('verify_new_login', 'button_cancel_verify_new', user_login, pass_login)
                
                return {'error': 'None'}
            else:
                return {'error': 'login_error'}

        elif form.getvalue("verify_new_login"):
            if self.test_input(form, 'verify_new_login') != None:
                user_login, pass_login = self.test_input(form, "verify_new_login")

                if {'user_login': user_login, 'pass_login': pass_login} == self.cookie_stored['temp_login']:
                    handler = {'error': 'None'} # 'hsid': hashlib.sha256(f'{self.current_cookie["hsid"]}'.encode()).hexdigest(), 
                    new_credentials = self.overwrite_credentials(user_login, pass_login)
                    self.html_login_form('login_changed', 'button_forgot', '', '')
                    self.log_message(f'INFO plcwebpage: Alterado credenciais de acesso ao webserver: \r\n{self.headers}')
                    self.empty_sessions()        # remover qualquer login anterior

                    try:
                        self.remove_id_session(globalVariables.path_session, self.current_cookie["sid"])
                    except:
                        self.log_message(f'INFO plcwebpage: Foi tentando remover uma sessao nao configurada: \r\n{self.headers}')

                else:
                    handler = {'error': 'login_not_equal'}
                    self.html_login_form('login_not_equal', 'button_cancel_new')

                return handler

            else:
                return {'error': 'login_error'}

        return {'error': 'login_error'}

    def test_input(self, form: FieldStorage, step):
        """
        sanitize input with invalid characters or blanck spaces
        """
        try:
            current_step = self.sanitize(form.getvalue(step))
            current_pass = self.sanitize(form.getvalue('Password'))
        except:
            return None
        # print(current_step, current_pass)
        return current_step, current_pass

    # change html forms for login steps
    def html_login_form(self, login_step, button_option, user_login=None, pass_login=None):
        """
        atualiza o formulario de login para realizar 
        validacao de credenciais e troca de senha
        """
        self.login_fields.update({'text_recovery': htmlText.text_recovery,
                'button_options': htmlText.button_options[button_option],
                'button_send_credentials': htmlText.button_send_credentials[button_option]})

        if user_login is None or pass_login is None:
            self.next_step(self.cookie_stored, self.current_cookie,
                        {"login_step": htmlText.login_dict[login_step]})
        else:
            self.next_step(self.cookie_stored, self.current_cookie,
                            {"login_step": htmlText.login_dict['verify_new_login'],
                            "temp_login": {'user_login': user_login, 'pass_login': pass_login}})

    def next_step(self, retrived, current_cookie, steps:dict):
        """
        atualiza o arquivo da sessao validada com os dados para
        alterar os html e dados enviados ao cliente
        """
        if current_cookie["sid"] in retrived: # verifica se contem SID antes
            for key, value in steps.items():
                self.write_data(f'{globalVariables.path_session}_{current_cookie["sid"]}', key, value)

