from webserver.variables import htmlText, globalVariables
from webserver.database import DataBase
from http.cookies import SimpleCookie
from  hashlib import sha256, new
import os
from time import time
from secrets import token_hex


class Session(DataBase):
    """
    -> classe para armazenar de maneira temporaria as sessoes e ids
    para melhor gerenciamento, realizando a desautorizacao e
    deletando usuarios nao desejaveis
    -> gerenciar todo credencial de sessoes,
    verificar se o navegador possui o correto cookie
    de sessao e se a senha correta foi inserida
    """
    def __init__(self, login="weg", password="weg") -> None:
        # caso existir outro login, sobrescrever admin
        self.login = login
        self.path_credentials = '/tmp/credentials'
        self.password = password
        self.cookie = SimpleCookie()
        # apenas para eliminar qualquer sessao anterior
        self.sid = None
    
    def __call__(cls, *args, **kwds):
        # return super().__call__(*args, **kwds)
        print("Chamado")
        cls.empty_sessions()

    def set_id_and_store(self, path):
        # cada acesso permitido deve possuir id unico
        self.sid = sha256(f'{token_hex()}'.encode()).hexdigest()
        file_path = f"{path}_{self.sid}"

        try:
            self.cookie.load({'sid': self.sid})
        except IOError:
            print('Erro na escrita da cookie')
        
        return self.write_data(file_path, f'{self.sid}', self.cookie)

    def retrive_id_cookie(self, path, sid) -> bool:
        cookie_path = f"{path}_{sid}"
        self.isValidCookie()
        return self.read_data(cookie_path)
    
    # verificar quando foi iniciada sessao
    def isValidCookie(self) -> bool:
        for file in os.listdir("/tmp"):
            if "session" == file[:len("session")]:
                file_time = os.stat("/tmp/" + file).st_ctime
                if (time() - file_time) >= 2000:
                    os.remove(f'/tmp/{file}')

    def remove_id_session(self, path, sid):
        # remover ID e permissoes de acesso
        self.remove_data(path, sid)
        # os.remove(f'{path}_{sid}')

    ######################## gerenciamento de sessoes #################

    def empty_sessions(self):
        print('Session removed')
        files = os.listdir("/tmp")
        for file in files:
            if "session" == file[:len("session")]:
                os.remove(f'/tmp/{file}')
        if 'logsPLC.zip' in files:
            os.remove('/tmp/logsPLC.zip')
        if 'secured.tar.gz' in files:
            os.remove('/tmp/secured.tar.gz')

    def hash_generator(self, hashstr):
        # apenas um generator para um novo hash
        yield new(hashstr)

    def create_session(self, key = 'weg', value = 'weg') -> dict:
        self.write_data(globalVariables.path_credentials, 'user', key)
        self.write_data(globalVariables.path_credentials, 'password',
                            self.hash_auth(value.encode()))
        return self.read_data(globalVariables.path_credentials)

    def hash_auth(self, auth):
        new_hash = next(self.hash_generator('sha256'))
        new_hash.update(bytes(auth))
        new_login = new_hash.hexdigest()
        return new_login

    def hash_compare_saved(self, credential) -> bool:
        system_login = self.check_login_exists()
        if credential[0] != system_login['user']:
            return False
        if self.hash_auth(credential[1].encode()) == system_login['password']:
            return True
        else:
            return False
    
    def check_login_exists(self):
        data = self.read_data(globalVariables.path_credentials)
        if not data:    # dicionario vazio
            return self.create_session()
        else:
            return data

    def overwrite_credentials(self, login = 'weg', password = 'weg') -> None:
        self.write_data(globalVariables.path_credentials, 'user', login)
        self.write_data(globalVariables.path_credentials, 'password',
                            self.hash_auth(password.encode()))
        data = self.read_data(globalVariables.path_credentials)
        os.system("plctools --log -logc")
        return data

    def forgot_pass(self, user, password) -> bool:
        new_hash = next(self.hash_generator('sha256'))
        new_hash.update(f'{htmlText.recovery_pass}'.encode())
        new_hash.update(htmlText.recover_string)
        if new_hash.hexdigest() == password and user == 'weg':
            return True
        else:
            return False