from webserver.variables import htmlText, urlsHandles, filesPaths, globalVariables, mutex
import os
from subprocess import Popen, PIPE, run
import shlex
from time import sleep
from ctypes import c_int8
from cgi import FieldStorage, MiniFieldStorage
from shutil import copyfileobj
import html
import re
# from socket import inet_aton, error
from zipfile import ZipFile
import tempfile


class Utilities(object):
    def __init__(self) -> None:
        self.path = ""

    def find_path(self, path, tipo):
        """
        encontra o caminho para o arquivo correspondente
        """
        # JSON retirados para serem executados pelo sistema
        if tipo in urlsHandles.TIPODICT.keys():
            self.files_path = os.getcwd()  \
                    + f"/bin/{urlsHandles.TIPODICT[tipo]}/"   # os.path.abspath('')

            if tipo == "application/json" and path != "/login.html":
                self.files_path = urlsHandles.json_paths[self.path]

            return self.files_path + path[1:]  # somar strings sem '/'

        else:
            raise KeyError
            # self.files_path = "None"

    def verify_exists(self, full_path) -> bool:
        """
        verificar se o caminho existe no projeto
        """
        # arquivo nao existe.
        if not os.path.exists(full_path):
            raise FileNotFoundError(f"'{self.path}' not found")

        # isso eh um arquivo valido
        elif os.path.isfile(full_path):
            return True

        # eh algum tipo nao suportado ou existente
        else:
            raise FileNotFoundError(f"Unknown object '{self.path}'")

    def handle_config(self, form: FieldStorage):
        """
        handle configuration changes
        TODO pass itens only
        # peguntar quando serao habilitadas as mudancas de ethernet e DHCP
        """
        if form.getvalue("SaveFTP") is not None:
            
            ftp_usb = form.getvalue('FTP-usb', 'off')
            ftp_sdcard = form.getvalue('FTP-sdcard', 'off')

            if ftp_usb == 'on':
                os.system("plctools --ftp usb 1")
            else:
                os.system("plctools --ftp usb 0")

            if ftp_sdcard == 'on':
                os.system("plctools --ftp sdcard 1")
            else:
                os.system("plctools --ftp sdcard 0")
            
            if form.getvalue('FTP'):
                self.exec_bash(f"{filesPaths.path_scripts['plctools']} --ftp type", htmlText.FTPtype[form.getvalue('FTP')])
                self.exec_bash(f"{filesPaths.path_scripts['initFTP']} ", '')
                ftp_value = form.getvalue('FTP')
                if ftp_value == 'Enable FTP':
                    os.system("plctools --log -ftpe")
                elif ftp_value == 'Enable FTPS':
                    os.system("plctools --log -ftps")
                else:
                    os.system("plctools --log -ftpd")

            if form.getvalue("Change-Password"):
                self.passw = form.getvalue("Change-Password")
                
                if self.no_special_characters(self.passw) and len(self.passw) <= 64:
                    self.exec_bash(f"{filesPaths.path_scripts['plctools']} --ftp pass", form.getvalue("Change-Password"))
                    self.exec_bash(f"{filesPaths.path_scripts['changepasswdFTP']} ", '')
                    os.system("plctools --log -ftpc")
                    return True

                return False

        elif form.getvalue("datetime_sync") is not None:
            values = form.getvalue("datetime_sync")
            os.system("plctools --log -clkb")
            self.exec_bash(f"{filesPaths.path_scripts['plctools']} -d", values)
            os.system("plctools --log -clks")

        elif form.getvalue("datetime_manual") is not None:
            values = form.getvalue("datetime_manual")
            os.system("plctools --log -clkm")
            self.exec_bash(f"{filesPaths.path_scripts['plctools']} -d", values)
            os.system("plctools --log -clks")

        elif form.getvalue("factoryreset") is not None:
            self.exec_bash(f"{filesPaths.path_scripts['factoryreset']}", "")

        elif form.getvalue("reboot") is not None:
            os.system("plctools --reboot")

        elif form.getvalue("SetIP") is not None:
            set_interface = form.getvalue("ETH")
            set_mode = form.getvalue("ETH-MODE")
            if set_interface == "eth1":
                if set_mode == "static": 
                    ip1 = form.getvalue("IP")
                    netmask1 = form.getvalue("NETMASK")
                    result = os.system(f"plctools -e 1 {ip1} {netmask1} 0")
                elif set_mode == "dhcp":
                    result = os.system(f"plctools -e 1 not_used not_used 1")
            elif set_interface == "eth2":
                if set_mode == "static": 
                    ip1 = form.getvalue("IP")
                    netmask1 = form.getvalue("NETMASK")
                    result = os.system(f"plctools -e 2 {ip1} {netmask1} 0")
                elif set_mode == "dhcp":
                    result = os.system(f"plctools -e 2 not_used not_used 1")
            globalVariables.temporary_ip = self.to_8_bits_bash_return(result)
            
    def no_special_characters(self, passw, pat=re.compile('[^a-zA-Z0-9]')) -> bool:
        if re.search(pat, passw):
            return False
        else:
            return True

    def file_download(self, form: FieldStorage):
        """
        salvando arquivos nas pastas correspondentes
        TODO melhorar fluxo 'try -> except'
        """
        with mutex:
            # apenas manipulando os arquivos recebidos pelo formulario
            for file in form:
                f = form[file]
                if( file == "file" ):
                    outpath = os.path.join(filesPaths.path_uploads['path_firm'], f.filename)
                else:
                    try:
                        os.listdir(filesPaths.path_uploads['path_app'])
                    except OSError:
                        os.makedirs(filesPaths.path_uploads['path_app'])
                    outpath = os.path.join(filesPaths.path_uploads['path_app'], f.filename)
                if form.getvalue(file) is not None:
                    if isinstance(form[file], list):
                        for f in form[file]:
                            self.recv_file(outpath, f)
                    elif form.getvalue(file) is not None:
                        f = form[file]
                        try:
                            self.recv_file(outpath, f)
                        except FileNotFoundError:
                            print("Diretorio nao existente ou sem permissao")

    # file_form deve retornar para assim executar esta funcao 
    def exec_update(self, keys:dict):
        """
        executar atulizacao d
        """
        if "file" in keys:
            update = self.exec_bash(filesPaths.path_scripts['plctools'], '-v')
            boot = None
        else:
            boot = self.exec_bash(filesPaths.path_scripts['plctools'], '--app')
            os.system("plctools --log -boot")
            update = None
        print(update, boot)
        return update, boot

    def exec_bash(self, script_path, args):
        with mutex:
            result = os.system(f"{script_path} {args}")
            sleep(1)    # nao realizar comandos em sequencia de maneira indefinida
        return self.to_8_bits_bash_return(result)

    def to_8_bits_bash_return(self, result):
        """
        retornando valor correspondente ao valor 8 bits (int) negativo
        """
        return int.from_bytes(c_int8(result >> 8), "little", signed=True)

    def commands_after_post(self, firm_update, boot_update, tipo):
        # print(firm_update, boot_update)
        if tipo == 'text/html':
            if firm_update == 0:
                self.exec_bash("(sleep 5; reboot) &", "")
                firm_update = 1
            if boot_update == 0:
                self.exec_bash("(sleep 5; reboot) &", "")
                boot_update = 1
    
        return firm_update, boot_update

    def recv_file(self, outpath, file: MiniFieldStorage) -> None:
        """jogar firmare em /opt/weg/codesys
        jogar aplicacao em /tmp/codesysApp/application"""
        with open(outpath, 'wb+') as path:
            copyfileobj(file.file, path, 100000)

    def sanitize(self, data_input:str):
        try:
            data_escape = html.escape(data_input, quote=True)
            data = data_escape.split()
            if isinstance(data, list) and len(data) >= 2:
                raise SyntaxError('Spaces are not accept in credenciais arguments!')

            data = data[0].encode('utf-8').decode()
            if len(data) > 64:
                raise ValueError('Too many characters! Limit to only 64 characters length')

        except SyntaxError as e:
            # self.value.update({'msgwarning': 'Error! White space or quotation marks are not acceptable'})
            return e
        except AttributeError as e:
            # self.value.update({'msgwarning': 'Error! {e}'})
            return e
        except ValueError as e:
            # self.value.update({'msgwarning': 'Error! {e}'})
            return e
        return data

    def zip_logs(self, path, param='log'):
        """
        realizar zip de arquivos de logs internos do sistema
        """
        path_zip = tempfile.mkstemp(suffix='.zip')
        print(path_zip)

        with ZipFile(path_zip[1], mode='w') as myzip:

            proc = run(['ls', path], capture_output=True)
            files = proc.stdout.decode().strip().split('\n')

            # filtro de arquivos para ZIP
            html_files = self.filter_string_list(files, param)
            # print(html_files)

            for file in html_files:
                # (abs_path, file_name) 
                myzip.write(path + file, os.path.basename(file))

        # print(path_zip)
        return path_zip

    def filter_string_list(self, string_list, param):
        return list(filter(lambda string: param in string, string_list)) # filter for desired files

    def encrypt_zip_file_openssl(self, file):
        path_ecrypt = '/tmp/secured_log.zip.enc'
        encrypt = "openssl aes-256-cbc -a -salt -pbkdf2 -k PASS -in FILE -out path_ecrypt"
        args = shlex.split(encrypt.replace('PASS', 'JARAGUA987654321').replace('FILE', file).replace('path_ecrypt', path_ecrypt))
        # print(args)
        with Popen(args, stdin=PIPE, stdout=PIPE) as proc:
            proc.wait()
        return path_ecrypt
    
    def remove_file(self, path):
        try:
            os.system('rm ' + path)
        except:
            pass