import uuid     # identificacao unica
import json
from threading import Lock
from os.path import exists
from time import sleep, perf_counter
from dataclasses import dataclass
from tempfile import NamedTemporaryFile
from shutil import copy


# controle de acesso R/W aos arquivos
mutex = Lock()

# copy to tmp file decorator
def copy_file(func):
    def copy_to_tmp(*args, **kwargs):
        tmp = NamedTemporaryFile(delete=True)  # tmp file context
        
        try:
            copy(args[0], tmp.name)    # tmp copy
            data = func(tmp.name)    # file data extract
        finally:
            tmp.close()  # deletes the file

        return data
    return copy_to_tmp

# decorate json read to get data from tmp file
@copy_file
def read_json(path):
    with open(path) as j:
        data = json.load(j)
    return data


@dataclass
class globalVariables:
    path_session = '/tmp/session'
    path_credentials = 'bin/login/credentials'
    firm_update = None
    boot_update = None
    temporary_ip = None

# pode ser utilizado para o terminal
@dataclass
class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

@dataclass
class additionalPages:
    INFOPAGE = '''<!DOCTYPE html>
        <html lang="en">
        <head>
            <title>INFO PAGE</title>
        </head>
        <body>
        <body>
        <table>
        <tr>  <td>Header</td>         <td>Info Page</td>      </tr>
        <tr>  <td>Date and time</td>  <td>{date_time}</td>    </tr>
        <tr>  <td>Client host</td>    <td>{client_host}</td>  </tr>
        <tr>  <td>Client port</td>    <td>{client_port}</td>  </tr>
        <tr>  <td>Command</td>        <td>{command}</td>      </tr>
        <tr>  <td>Path</td>           <td>{path}</td>         </tr>
        </table>
        </body>
        </html>'''

#    ERRORPAGE = '''<!DOCTYPE html>
#        <html lang="en">
#        <head>
#            <title>ERROR PAGE</title>
#        </head>
#        <body>
#        <h1>Access Error {path}</h1>
#        <p>{msg}</p>
#        <a href="login.html">Return to initial page</a>
#        </body>
#        </html>
#        '''

@dataclass
class filesPaths:
    # verificar URLs permitidas
    files_zip = {
        '/getcodesyslog': '/var/lib/weg/codesys/', # *CSV
        '/getdevlog': '/var/log/weg/', # *
        # '/gettestlog': '/home/lucas/Downloads/'
    }

    log_files_extension = {
        '/getcodesyslog': 'csv',
        '/getdevlog': 'log'
        # '/gettestlog': '/home/lucas/Downloads/'
    }

    path_scripts = {
        'start': '/opt/weg/codesys/',
        'factoryreset': '/opt/weg/scripts/plcfacReset.sh',
        'plctools': 'plctools',
        'changepasswdFTP':'/opt/weg/scripts/ftp/changepasswdFTP.sh',
        'initFTP': '/opt/weg/scripts/ftp/initFTP.sh',
        'plcsecrets': '/opt/weg/scripts/plcsecrets.sh'
        }

    path_uploads = {'path_firm':"/var/lib/weg/codesys",
                    'path_app': "/tmp/codesysApp/application"}

@dataclass
class urlsHandles:
    URLS = {
        '/',
        '/getcodesyslog', 
        '/getdevlog', 
        # "/changelogin.html",
        # "/error.html",
        "/status.html",
        "/configuration.html",
        "/visualization.html",
        "/firmwareupdate.html",
        "/login.html"
        }


    # URL permitidos para acessar sem credenciais
    URLS_ALLOWED_ALWAYS = {
        '/login.html',
        '/bootstrap.min.css',
        '/jquery-3.6.0.min.js',
        '/style.css',
        '/bootstrap.min.js',
        '/weg-logo.png',
        '/favicon.ico',
        '/login.js',
        '/plchash.json'
    }

    # indicando caminho para cada tipo de arquivo
    TIPODICT = {
        "text/html": "www", None: "www", "text/javascript": "js",
        "image/x-icon": "figures", "application/json": "json",
        "application/javascript": "js", "text/plain": "txt",
        "image/vnd.microsoft.icon": "figures",
        "text/css": "css", "image/png": "figures"
        }

    # dicionario para indicar caminho de cada json
    json_paths = {
        "/cpu.json": "/run/weg/json/",
        "/dataInfo.json": "/run/weg/json/",
        "/networks.json": "/run/weg/json/",
        "/gpios.json": "/run/weg/json/",
        "/ftpcfg.json": "/etc/weg/ftp/",
        "/firstBoot.json": "/etc/weg/",
        "/plchash.json": "/etc/weg/",
    }

@dataclass
class htmlText:
    FTPtype = {'Disable': 0,
        'Enable FTP': 1,
        'Enable FTPS': 2}

    # campos permitidos dos formularios, qualquer outro sendo ignorado
    form_fields = ['Login', 'Logout', 'Password', 
                    'Change', 'Cancel', 
                    'login_verify', 'new_login', 'verify_new_login', 
                    'factoryreset', 'reboot','datetime_sync', 'datetime_manual', 'file', 'appfile', 'crcfile', 
                    'FTP', 'Change-Password', 'FTP-sdcard', 'FTP-usb', 'reinstall', 'SaveFTP',
                    'IP', 'NETMASK', 'ETH', 'ETH-MODE', 'SetIP']

    # strings para auxilio e direcionamento da troca de crendenciais de acesso
    login_dict = {
        'Login':{
            'msgwarning':'',
            'login_placeholder':'Login',
            'msgerror':'',
            'PageType':'LOGIN PAGE',
            'TextUsername': 'Username',
            'TextPassword': 'Password'
        },
        'login_verify': {
            'msgwarning':'',
            'login_placeholder':'login_verify',
            'msgerror':'',
            'PageType':'CHANGE LOGIN',
            'TextUsername': 'Username',
            'TextPassword': 'Password'
        },
        'login_verify_error': {
            'msgwarning':'',
            'login_placeholder':'login_verify',
            'msgerror': 'Invalid credential',
            'PageType':'CHANGE LOGIN',
            'TextUsername': 'Username',
            'TextPassword': 'Password'
        },
        'new_login':{
            'msgwarning':'',
            'login_placeholder':'new_login',
            'msgerror':'',
            'PageType':'CHANGE LOGIN',
            'TextUsername': 'New Username',
            'TextPassword': 'New Password'
        },
        'verify_new_login':{
            'msgwarning':'',
            'login_placeholder':'verify_new_login',
            'msgerror':'',
            'PageType':'CHANGE LOGIN',
            'TextUsername': 'Confirm Username',
            'TextPassword': 'Confirm Password'
        },
        'login_error':{
            'msgwarning':'',
            'login_placeholder':'Login',
            'msgerror': 'Invalid credential',
            'PageType':'LOGIN PAGE',
            'TextUsername': 'Username',
            'TextPassword': 'Password'
        },
        'login_changed':{
            'msgwarning':'',
            'login_placeholder':'Login',
            'msgerror': '',
            'PageType':'CHANGE LOGIN',
            'TextUsername': 'Username',
            'TextPassword': 'Password'
        },
        'login_not_equal':{
            'msgwarning':'',
            'login_placeholder':'new_login',
            'msgerror': 'Provided credentials were not equal!',
            'PageType':'CHANGE LOGIN',
            'TextUsername': 'New Username',
            'TextPassword': 'New Password'
        },
        'change_login_error':{
            'msgwarning':'',
            'login_placeholder':'login_verify',
            'msgerror': 'Invalid credential',
            'PageType':'CHANGE LOGIN',
            'TextUsername': 'Username',
            'TextPassword': 'Password'
        }
    }

    # botao para envio de credenciais
    button_send_credentials = {'button_forgot': """<button type="submit" 
                            class="btn btn-lg btn-primary px-4" id="login-login">
                            Login</button>""",
                        'button_cancel_verify':"""<button type="submit" 
                            class="btn btn-lg btn-primary px-4" id="login-login">
                            Verify Login
                            </button>""",
                        'button_cancel_new':"""<button type="submit" 
                            class="btn btn-lg btn-primary px-4" id="login-login">
                            New Login</button>""",
                        'button_cancel_verify_new':"""<button type="submit" 
                            class="btn btn-lg btn-primary px-4" id="login-login">
                            Confirm Login</button>"""}

    # botao para cancelar recuperacao de senha
    button_options = {'button_forgot': """<button type="button" class="btn btn-lg btn-secondary px-4" id="forget-password" name="Forgot"
                            value="Forgot" onclick='p = document.getElementById("p"); p.classList.toggle("hideP");'>
                            Support</button>""",
                    'button_cancel_verify':"""<button type="submit" class="btn btn-lg btn-danger px-4" id="forget-password" 
                        name="Cancel" value="1" formnovalidate> Cancel</button>""",
                    'button_cancel_new':"""<button type="submit" class="btn btn-lg btn-danger px-4" id="forget-password" 
                        name="Cancel" value="1" formnovalidate> Cancel</button>""",
                    'button_cancel_verify_new':"""<button type="submit" class="btn btn-lg btn-danger px-4" id="forget-password" 
                        name="Cancel" value="1" formnovalidate> Cancel</button>"""}

    # Ler o recovery_pass do arquivo
    recovery_pass = "file not found"
    login_fields = {'recovery_pass': recovery_pass}

    # txto para recuperacao de senha com recovery
    text_recovery = """<div class="d-flex justify-content-center mt-2">
                        <p id="p" class="hideP" title="A new code will be generated every system reboot." style="font-size: 9pt;">
                            - The credentials can be reset through PLC Shell in Codesys.
                            <br>- Support hash:
                            <b id="hash">{recovery_pass}</b>
                        </p>
                    </div>""".format(**login_fields)

    # txto para recuperacao de senha
    #text_recovery = """<div class="d-flex justify-content-center mt-2">
    #                    <p id="p" class="hideP" title="A new code will be generated every system reboot." style="color: #ff0000;">
    #                        Contact WEG technical assistance.
    #                    </p>
    #                </div>""".format(**login_fields)

    # TODO remover alteracao  da criacao de dados
    # codigo acoplado e pouca flexibilidade
    truth = []
    t = perf_counter()
    
    while not any(truth):
        for file in urlsHandles.json_paths:
            truth.append(exists(urlsHandles.json_paths[file] + file[1:]))
        sleep(5)
    pc = perf_counter() - t

    # if sys.platform != 'win32':
    dataInfo_file = '/dataInfo.json'
    try:
        with open(urlsHandles.json_paths[dataInfo_file] + dataInfo_file[1:], 'rb') as f:
            dataInfo = json.load(f)
            print(dataInfo)
            login_fields.update({'WebpagePLC': dataInfo['HWVERSION']})
            login_fields.update({'app_version': dataInfo['FWVERSION']})
    except FileNotFoundError:
        login_fields.update({'WebpagePLC': f'PLC'})
        login_fields.update({'app_version': 'firmver'})

    login_fields.update(login_dict['Login'])
    login_fields.update({'text_recovery': text_recovery,
                        'button_options': button_options['button_forgot'],
                        'button_send_credentials': button_send_credentials['button_forgot']})

            
    recover_string = b'weg1234' # talvez salvar como variavel de ambiente